#!/bin/env python
################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
farm.py - execute a cmd on the cluster
======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   cat go | farm.py --split-at-colum=1 perl -p -e "s/GO/gaga/"

Type::

   python farm.py --help

for command line help.

Documentation
-------------

The input on stdin is split for embarrasingly parallel jobs.
The --split-at-.. options describe how standard input is to
be split. A temporary directory is created in the current
directory. This directory has be visible on the cluster nodes
and accessible under the same name.

The output is written to stdout. Results are returned in the
same order as they are submitted.

On error, error messages are echoed and nothing is returned.
The temporary directory is not deleted to allow manual recovery.

Examples
--------

The following command will split the file "go" at the first column 
and execute the command perl -p -e "s/GO/gaga/"::

   cat go | farm.py --split-at-colum=1 perl -p -e "s/GO/gaga/"

The following command will split a fasta file at each entry and
compute an approximate sequence length::

   cat genome.fasta | farm.py --split-at-regex="^>(\S+)" "wc -c"

The following command will split a fasta file at every 10 sequences::

   cat genome.fasta | farm.py --split-at-regex="^>(\S+)" --chunksize=10 "wc -c"

.. todo::

   implement continuation of jobs
   implement better error messages
   use sge array jobs for job control

Code
----

'''

import os
import sys
import re
import string
import optparse
import time
import glob
import subprocess
import tempfile
import shutil
import stat

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import threadpool
import multiprocessing

try:
    import drmaa
    HASDRMAA = True
except ImportError:
    HASDRMAA = False


def chunk_iterator_lines( infile, args, prefix, use_header = False ):
    """split by lines."""

    chunk_size = args[0]
    n = 0
    filename = "%s/%010i.in" % (prefix, n)
    outfile = IOTools.openFile( filename, "w" )
    header = None

    for line in infile:
        if line[0] == "#": continue

        if not header and n == 0 and use_header:
            header = line
            outfile.write( header )
            continue

        n += 1

        if n % chunk_size == 0:
            outfile.close()
            yield filename
            filename = "%s/%010i.in" % (prefix, n)
            outfile = IOTools.openFile( filename, "w" )
            if header: outfile.write( header )

        outfile.write( line )
    outfile.close()
    yield filename

def chunk_iterator_column( infile, args, prefix, use_header = False ):
    """split at column. 

    The table need not be sorted by this column.
    If num_files is given, files will randomly created
    and tags according to column randomly assigned.


    """

    column, max_files = args
    files = IOTools.FilePool()
    header = False

    if max_files:
        map_tag2file = {}

    for line in infile:
        if line[0] == "#": continue

        if not header and use_header: 
            files.setHeader( line )
            header = True
            continue

        key = line[:-1].split("\t")[column]
        if max_files:
            if key in map_tag2file:
                key= map_tag2file[key]
            else:
                n= "%010i" % (len(map_tag2file) % max_files)
                map_tag2file[key] = n
                key = n

        files.write( "%s/%s.in" % (prefix, key), line )

    for filename, count in files.items():
        E.info( "created file %s with %i items" % (filename, count ) )
        yield filename

def chunk_iterator_regex_group( infile, args, prefix, use_header = False ):
    """group by regular expression is true.

    Entries need to be consecutive.
    """

    rex = args[0]
    column = args[1]
    chunk_size = args[2]
    last = None
    header = None
    n = chunk_size
    outfile = None

    for line in infile:

        if line[0] == "#": continue

        if not header and use_header: 
            header = line
            continue

        try:
            this = rex.search(line[:-1]).groups()[0]
        except IndexError:
            if outfile: outfile.write(line)
            continue
        except AttributeError:
            if outfile: outfile.write(line)
            continue

        if last != this and n >= chunk_size:
            if last:
                outfile.close()
                yield filename

            last = this

            filename = "%s/%s.in" % (prefix, this)
            outfile = IOTools.openFile( filename, "w" )
            if header: outfile.write( header )
            n = 0

        outfile.write( line )
        n += 1

    if outfile:
        outfile.close()
        yield filename

def chunk_iterator_regex_split( infile, args, prefix, use_header = False ):
    """split where regular expression is true.
    """

    rex = args[0]
    column = args[1]
    chunk_size = args[2]
    max_lines = args[3]
    last = None
    header = None

    nlines = 0
    n = 0
    filename = "%s/%010i.in" % (prefix, n)
    outfile = IOTools.openFile( filename, "w" )

    for line in infile:

        if line[0] == "#": continue

        if rex.search(line[:-1]):
            if n > 0 and (n % chunk_size == 0 or (max_lines and nlines > max_lines)):
                outfile.close()
                yield filename
                filename = "%s/%010i.in" % (prefix, n)
                outfile = IOTools.openFile( filename, "w" )
                nlines = 0

            n += 1
                
        outfile.write( line )
        nlines += 1

    outfile.close()
    yield filename

def chunk_iterator_psl_overlap( infile, args, prefix, use_header = False ):
    """iterate over overlapping entries in a psl file."""

    iterator = Blat.BlatIterator( sys.stdin )

    processed_contigs = set()

    merge_distance=args[0]

    while 1:

        match = iterator.next()
        
        if match == None: break
        
        if match.mSbjctId != last_sbjct_id or match.mSbjctFrom >= (sbjct_end + merge_distance):
            if last_sbjct_id:
                outfile.close()
                yield filename

            if last_sbjct_id != match.mSbjctId and match.mSbjctId in processed_contigs:
                raise ValueError("input not sorted correctly (contig,start): already encountered %s\n%s" % (match.mSbjctId, str(match)))

            last_sbjct_id = match.mSbjctId
            processed_contigs.add( last_sbjct_id )

            sbjct_start = match.mSbjctFrom
            sbjct_end = match.mSbjctTo
            
        if match.mSbjctFrom < sbjct_start:
            raise ValueError("input not sorted correctly (contig,start): %i < %i\n%s" % (match.mSbjctFrom, sbjct_start, str(match)))

        sbjct_end = max( match.mSbjctTo, sbjct_end )
        outfile.write( str(match)+"\n" )

    if outfile:
        outfile.close()
        yield filename

class MapperGlobal:
    def __init__(self, pattern="%06i"):
        self.mMap = {}
        assert "%" in pattern, "please supply a pattern"
        self.mPattern = pattern

    def __call__(self, fn, id ):
        if id not in self.mMap:
            self.mMap[id] = self.mPattern % (len(self.mMap) + 1)
        return self.mMap[id]

class MapperLocal:
    def __init__(self, pattern="%06i"):
        self.mMap = {}
        assert "%" in pattern, "please supply a pattern"
        self.mPattern = pattern

    def __call__(self, fn, id ):
        key = "%s-%s" % (fn, id)
        if key not in self.mMap:
            self.mMap[key] = self.mPattern % (len(self.mMap) + 1)
        return self.mMap[key]

class MapperEmpty:
    def __init__(self):
        pass
    def __call__(self, fn, id ):
        return id

class ResultBuilder:
    """the default result builder for table formatted output."""

    def __init__( self, mapper = None, field_index = None, field_name = None):
        self.mMapper = mapper
        self.mFieldIndex = field_index
        self.mFieldName = field_name
        self.mHeader = None

    def parseHeader( self, infile, outfile, options ):
        """parse header in infile."""
        # skip comments until header
        while 1:
            l = infile.readline() 
            if not l or l[0] != "#": break
            options.stdlog.write(l)

        # Print only the first header and check if
        # all the headers are the same.
        if self.mHeader:
            if self.mHeader != l:
                raise ValueError( "inconsistent header in file %s\ngot=%s\nexpected=%s" % (infile, l, self.mHeader) )
        else:
            outfile.write( l )
            self.mHeader = l

            if self.mFieldIndex == None and self.mFieldName:
                try:
                    self.mFieldIndex = self.mHeader.split("\t").index( self.mFieldName )
                except ValueError:
                    if options.loglevel >= 1:
                        options.stdlog.write("# Warning: no mapping, can not find field %s in %s" % (self.mFieldName, self.mHeader) )
                    self.mFieldName = None
            
                if options.loglevel >= 2:
                    options.stdlog.write("# substituting field: %s, %s\n" % (self.mFieldName, self.mFieldIndex))

    def __call__( self, filenames, outfile, options ):

        for fi, fn in filenames:
            E.debug( "# merging %s" % fn )
            infile = IOTools.openFile( fn, "r" )

            if options.output_header: 
                self.parseHeader( infile, outfile, options )

            for l in infile:
                if l[0] == "#":
                    options.stdlog.write(l)
                else:
                    if self.mFieldIndex != None:
                        data = l[:-1].split("\t")
                        try:
                            data[self.mFieldIndex] = self.mMapper( fi, data[self.mFieldIndex] )
                        except IndexError:
                            raise IndexError( "can not find field %i in %s" % (self.mFieldIndex, l ) )
                        l = "\t".join( data ) + "\n"
                        
                    outfile.write(l)
            infile.close()

class ResultBuilderPSL ( ResultBuilder ):
    """Result builder for psl tables. Here, column 9, the query id, is substituted."""

    def __init__( self, *args, **kwargs):
        ResultBuilder.__init__(self, *args, **kwargs)
        self.mFieldIndex = 9
        self.mFirst = True

    def parseHeader( self, infile, outfile, options ):
        """parse header in infile."""
        # skip comments until header
        while 1:
            l = infile.readline() 
            if not l or l[0] != "#": break
            options.stdlog.write(l)

        if l.startswith( "psLayout version 3"):
            if self.mFirst:
                outfile.write( l )
                for x in range(0,4):
                    l = infile.readline()
                    outfile.write( l )
                self.mFirst = False
            else:
                for x in range(0,4):
                    l = infile.readline()

class ResultBuilderFasta( ResultBuilder) :

    def __init__( self, *args, **kwargs ):
        ResultBuilder.__init__(self, *args, **kwargs)
    
    def __call__( self, filenames, outfile, options ):
        for fi, fn in filenames:
            infile = IOtools.openFile( fn, "r" )
            for l in infile:
                if l[0] == "#": 
                    options.stdlog.write( l )
                    continue
                elif l[0] == ">":
                    x = re.search(">(\S+)", l[:-1])
                    id = self.mMapper( fi, x.groups()[0] )
                    l = ">%s%s" % (id, l[x.end(0):])
                outfile.write(l)
            infile.close()

class ResultBuilderBinary( ResultBuilder) :
    '''simply concatenate output files (without any parsing).'''

    def __init__( self, *args, **kwargs ):
        ResultBuilder.__init__(self, *args, **kwargs)
    
    def __call__( self, filenames, outfile, options ):
        for fi, fn in filenames:
            shutil.copyfileobj( IOTools.openFile(fn, "r"), outfile )

class ResultBuilderCopies( ResultBuilder) :
    '''create indexed copiers.'''

    def __init__( self, *args, **kwargs ):
        ResultBuilder.__init__(self, *args, **kwargs)
    
    def __call__( self, filenames, outfile, options ):
        idx = 0
        base, ext = os.path.splitext( outfile.name )
        for fi, fn in filenames:
            idx += 1
            shutil.copyfile( fn, base + ".%i" % idx + ext )

class ResultBuilderLog( ResultBuilder) :
    """processor for log files."""

    def __init__( self, *args, **kwargs ):
        ResultBuilder.__init__(self, *args, **kwargs)
    
    def __call__( self, filenames, outfile, options ):
        for fi, fn in filenames:
            infile = IOTools.openFile( fn, "r" )
            outfile.write("######### logging output for %s ###################\n" % fi)
            for l in infile:
                outfile.write(l)
            infile.close()

##--------------------------------------------------------------------
def runCommand( data ):

    filename, cmd, options, tmpdir, subdirs = data

    if subdirs:
        outdir = "%s.dir/" % (filename)
        os.mkdir( outdir )
        cmd = re.sub( "%DIR%", outdir, cmd )

    x = re.search( "'--log=(\S+)'", cmd ) or re.search( "'--L\s+(\S+)'", cmd )
    if x:
        logfile = filename + ".log" 
        cmd = cmd[:x.start()] + "--log=%s" % logfile + cmd[x.end():]
    else:
        logfile = filename + ".out"

    # working directory - needs to be the one from which the
    # the script is called to resolve input files.
    cwd = os.getcwd()

    if "<(" in cmd or "|" in cmd:
        if "'" in cmd: raise ValueError( "advanced bash syntax `<()` combined with single quotes" )
        cmd = """/bin/bash -c '%s'""" % cmd

    if "|" in cmd:
        if r"\|" not in cmd:
            E.warn( "pipes (`|`) within command need to be escaped, otherwise jobs run on submit host" )

    c = '%s -v "BASH_ENV=%s" -q %s -p %i %s %s' % (options.cluster_cmd, 
                                                   options.bashrc,
                                                   options.cluster_queue,
                                                   options.cluster_priority,
                                                   options.cluster_options,
                                                   cmd )

    iteration = 0

    while 1:

        iteration += 1
        if iteration > 1:
            E.info("%s: re-submitting command (repeat=%i): %s" % (filename, iteration, c ))
        else:
            E.info("%s: submitting command: %s" % (filename, c))

        infile = IOTools.openFile( filename, "r" )
        outfile = IOTools.openFile( filename + ".out", "w")
        errfile = IOTools.openFile( filename + ".err", "a")

        retcode = subprocess.call( c,
                                   shell = True,
                                   stdin = infile,
                                   stdout = outfile,
                                   stderr = errfile,
                                   cwd = cwd,
                                   close_fds = True)                              

        infile.close()
        outfile.close()
        errfile.close()

        if hasFinished( retcode, filename, options.output_tag, logfile ):
            break

        if iteration > options.resubmit:
            E.warn( "%s: giving up executing command: retcode=%i" % (filename, retcode) )
            break

        E.warn( "%s: error while executing command: retcode=%i" % (filename, retcode) )


    return (retcode, filename, cmd, logfile, iteration)

##--------------------------------------------------------------------
def hasFinished( retcode, filename, output_tag, logfile ):
    """check if a run has finished."""

    E.info( "checking status of job %s with returncode %i" % (filename, retcode) )
    if retcode != 0: 
        try:
            if not output_tag or not re.search( output_tag, IOTools.getLastLine( logfile )):
                return False
        except IOError:
            E.warn( "could not read output_tag from files %s" % (logfile) )
            return False
    return True


##--------------------------------------------------------------------
def runDRMAA( data, environment ):
    '''run jobs in data using drmaa to connect to the cluster.'''
    # prefix to detect errors within pipes
    prefix = '''detect_pipe_error_helper() 
    {
    while [ "$#" != 0 ] ; do
        # there was an error in at least one program of the pipe
        if [ "$1" != 0 ] ; then return 1 ; fi
        shift 1
    done
    return 0 
    }
    detect_pipe_error() {
    detect_pipe_error_helper "${PIPESTATUS[@]}"
    return $?
    }
    '''
    suffix = "; detect_pipe_error"
    
    # working directory - needs to be the one from which the
    # the script is called to resolve input files.
    cwd = os.getcwd()

    session = drmaa.Session()            
    session.initialize()
    jt = session.createJobTemplate()
    
    jobids = []
    kwargs = {}

    for filename, cmd, options, tmpdir, subdirs in data:

        from_stdin, to_stdout = True, True

        if subdirs:
            outdir = "%s.dir/" % (filename)
            os.mkdir( outdir )
            cmd = re.sub( "%DIR%", outdir, cmd )

        x = re.search( "'--log=(\S+)'", cmd ) or re.search( "'--L\s+(\S+)'", cmd )
        if x:
            logfile = filename + ".log" 
            cmd = cmd[:x.start()] + "--log=%s" % logfile + cmd[x.end():]
        else:
            logfile = filename + ".out"

        if "%STDIN%" in cmd:
            cmd = re.sub("%STDIN%", filename, cmd )
            from_stdin = False

        if "%STDOUT%" in cmd:
            cmd = re.sub("%STDOUT%", filename + ".out", cmd )
            to_stdout = False

        cmd = " ".join( re.sub( "\t+", " ", cmd).split( "\n" ) )
        E.debug( "running statement:\n%s" % cmd )

        tmpfile = tempfile.NamedTemporaryFile( dir = os.getcwd() , delete = False )
        tmpfile.write( "#!/bin/bash\n" ) #  -l -O expand_aliases\n" )
        tmpfile.write( " ".join( (prefix, cmd, suffix)) + "\n" )
        tmpfile.close()

        job_path = os.path.abspath( tmpfile.name )

        os.chmod( job_path, stat.S_IRWXG | stat.S_IRWXU )

        # get session for process - only one is permitted
        jt.workingDirectory = os.getcwd()
        jt.remoteCommand = job_path
        e = { 'BASH_ENV' : options.bashrc }
        if environment:
            for en in environment:
                try:
                    e[en] = os.environ[en]
                except KeyError:
                    raise KeyError("could not export environment variable '%s'" % en )
        jt.jobEnvironment = e

        jt.args = []
        jt.nativeSpecification = "-V -q %s -p %i -N %s %s" % \
            (kwargs.get("job_queue", options.cluster_queue ),
             kwargs.get("job_priority", options.cluster_priority ),
             os.path.basename(kwargs.get("outfile", "farm.py" )),
             kwargs.get("job_options", options.cluster_options))
        
        # keep stdout and stderr separate
        jt.joinFiles=False

        # use stdin for data
        if from_stdin: jt.inputPath=":" + filename

        # later: allow redirection of stdout and stderr to files
        # could this even be across hosts?
        if to_stdout: 
            jt.outputPath=":"+ filename + ".out"
        else:
            jt.outputPath=":"+ filename + ".stdout"

        jt.errorPath=":" + filename + ".err"


        jobid = session.runJob(jt)
        jobids.append( (jobid, job_path, filename, cmd, logfile) )

    E.debug( "%i jobs have been submitted" % len(jobids) )

    results = []

    for jobid, job_path, filename, cmd, logfile in jobids:

        try:
            retval = session.wait(jobid, drmaa.Session.TIMEOUT_WAIT_FOREVER)
        except Exception, msg:
            # ignore message 24 in PBS
            # code 24: drmaa: Job finished but resource usage information and/or termination status could not be provided.":
            if not msg.message.startswith("code 24"): raise
            retval = None

        if retval and retval.exitStatus != 0:
            raise OSError( "Child was terminated by signal %i: \n%s\n" % \
                               (retval.exitStatus, cmd))
        
        results.append( (retval, filename, cmd, logfile, 1 ) )

        os.unlink( job_path )

    session.deleteJobTemplate(jt)
    session.exit()

##--------------------------------------------------------------------
def getOptionParser():
    """create parser and add options."""

    parser = E.OptionParser( version = "%prog version: $Id: farm.py 2782 2009-09-10 11:40:29Z andreas $", 
                                    usage = globals()["__doc__"] )


    parser.add_option( "--split-at-lines", dest="split_at_lines", type="int",
                       help = "split jobs according to line number [default=%default]." )

    parser.add_option( "--split-at-column", dest="split_at_column", type="int",
                       help = "split jobs according to column. Columns start at number 1 and the input should be sorted by this column [default=%default]." )

    parser.add_option( "--group-by-regex", dest="group_by_regex", type="string",
                       help = "group jobs according to a regular expression [default=%default]." )

    parser.add_option( "--split-at-regex", dest="split_at_regex", type="string",
                       help = "split jobs according to a regular expression [default=%default]." )

    parser.add_option( "--split-at-tag", dest="split_at_tag", type="int",
                       help = "split a file at a tag [default=%default]." )

    parser.add_option( "--chunksize", dest="chunksize", type="int",
                       help = "when splitting at regex or tag, aggregate x entries [default=%default]." )

    parser.add_option( "--debug", dest="debug", action="store_true",
                       help = "debug mode. Do not delete temporary file [default=%default]." )

    parser.add_option( "--dry-run", dest="dry_run", action="store_true",
                       help = "dry run. Do not split input and simply forward stdin to stdout - " 
                              "useful for debugging the command [default=%default]." )

    parser.add_option( "--input-header", dest="input_header", action="store_true",
                       help = "The input stream contains a table header. " 
                              "This header is replicated for each job [default=%default]." )

    parser.add_option( "--output-header", dest="output_header", action="store_true",
                       help = "The output jobs contain a table header. " 
                              "The header is removed for each job except for the first [default=%default]." )

    parser.add_option( "--output-tag", dest="output_tag", type="string",
                       help = "The output jobs contain a tag in the last line denoting job completion. If the unix return value denotes an error, the presence of this tag is checked [default=%default]." )

    parser.add_option( "--subdirs", dest="subdirs", action="store_true",
                       help = "Run within separate subdirs for jobs. This permits multiple output streams. Use a placeholder %DIR% if you supply the ouput pattern as a command line option [default=%default]." )

    parser.add_option( "-T", "--tmpdir", dest="tmpdir", type="string",
                       help = "Temporary directory to be used. Default is the current directory [default=%default]." )
    
    parser.add_option( "--max-files", dest="max_files", type="int",
                       help = "create at most x files [default=%default]." )

    parser.add_option( "--max-lines", dest="max_lines", type="int",
                       help = "in addition to splitting into chunksize, also split if more than max-lines is reached [default=%default]." )

    parser.add_option( "--renumber", dest="renumber", type="string",
                       help = "renumber ids consecutively, supply a pattern [default=%default]." )

    parser.add_option( "--renumber-column", dest="renumber_column", type="string", action="append",
                       help = "specify column to renumber. The format is regex:column, for example csv:1 or csv:id [default=%default]." )
    
    parser.add_option( "-r", "--reduce", dest="reduce", type="string", action="append",
                       help = "Add reduce functions for specific files. The format is file:reducer. The default reducer is 'table' for all files [default=%default]." )
    
    parser.add_option( "-m", "--map", dest="map", type="string", action="append",
                       help = "Map specific columns in tables. The format is file:column:pattern, for example .table:1:%06i [default=%default]." )

    parser.add_option( "--resume", dest="resume", type ="string",
                       help = "resume aborted run from files in dir [%default]")

    parser.add_option( "--collect", dest="collect", type ="string",
                       help = "collect files in dir and process as normally [%default]")

    parser.add_option( "--binary", dest="binary", action="store_true",
                       help = "the output is binary - files are concatenated without parsing [%default]")

    parser.add_option( "--resubmit", dest="resubmit", type="int",
                       help = "if a job fails, automatically resubmit # times. Set to 0 in order to disable resubmission [%default]")

    parser.add_option( "--fail", dest="resubmit", action="store_false",
                       help = "if a job fails, do not resubmit [%default]")

    parser.add_option( "--bashrc", dest="bashrc", type="string",
                       help = "bashrc file to use [%default]")

    parser.add_option( "--method", dest="method", type="choice",
                       choices=("multiprocessing", "threads", "drmaa"),
                       help = "method to submit jobs [%default]")

    parser.add_option( "-e", "--env", dest="environment", type="string", action="append",
                       help = "environment variables to be passed to the jobs [%default]" )

    parser.add_option( "--output-pattern", dest="output_pattern", type="string",
                       help = "Pattern for secondary output filenames. Should contain a '%s' " 
                              "[default=%default]." )
    parser.set_defaults( 
        split_at_lines = None,
        split_at_column = None,
        split_at_regex = None,
        group_by_regex = None,
        split_at_tag = None,
        chunksize = None,
        cluster_cmd = 'qrsh -cwd -now n',
        bashrc = "~/.bashrc",
        input_header = False,
        output_header = False,
        debug = False,
        dry_run = False, 
        tmpdir = "./",
        subdirs = False,
        renumber = None,
        output_tag = "# job finished",
        map = [], 
        reduce = [],
        resume = None,
        renumber_column = [],
        resubmit = 5, 
        collect = None, 
        method = "drmaa",
        max_files = None,
        max_lines = None,
        binary = False,
        environment = [],
        output_pattern = "%s",
        )

    ## stop parsing options at the first argument
    parser.disable_interspersed_args()

    return parser
    
##--------------------------------------------------------------------
def main():

    parser = getOptionParser()

    (options, args) = E.Start( parser, 
                               add_cluster_options = True )

    if len(args) == 0:
        raise ValueError( "command line argument missing - see usage information" )

    options.renumber_column = [ x.split( ":" ) for x in options.renumber_column ]

        
    cmd = args[0]
    if len(args) > 1:
        cmd += " '" + "' '".join(args[1:]) + "'" 

    if options.dry_run:

        cmd = re.sub( "%DIR%", "", cmd )
        retcode = subprocess.call( cmd,
                                   shell = True,
                                   stdin = sys.stdin,
                                   stdout = sys.stdout,
                                   cwd = os.getcwd(),
                                   close_fds = True)                              
        E.Stop()
        sys.exit(0)

    failed_requests = []
    started_requests = []
    niterations = 0

    if not options.collect:
        tmpdir = os.path.abspath( tempfile.mkdtemp( dir = options.tmpdir ) )

        E.info( " working in directory %s" % tmpdir )

        if options.split_at_lines:
            chunk_iterator = chunk_iterator_lines
            args = (options.split_at_lines,)
        elif options.split_at_column:
            chunk_iterator = chunk_iterator_column
            args = (options.split_at_column - 1, options.max_files )
        elif options.split_at_regex:
            chunk_iterator = chunk_iterator_regex_split
            args = (re.compile( options.split_at_regex ),0,options.chunksize, options.max_lines )
        elif options.group_by_regex:
            chunk_iterator = chunk_iterator_regex_group
            args = (re.compile( options.group_by_regex ),0, options.chunksize)
        else:
            raise ValueError( "please specify a way to chunk input data" )

        data = [ (x, cmd, options, None, options.subdirs) for x in chunk_iterator( options.stdin, 
                                                                                   args, 
                                                                                   prefix = tmpdir,
                                                                                   use_header = options.input_header ) ]

        started_requests = [ (x[0], x[0]+ ".out") for x in data ]

        if len(data) == 0:
            E.warn("no data received")
            E.Stop()
            sys.exit(0)

        if options.method == "multiprocessing":
            pool = multiprocessing.Pool( options.cluster_num_jobs )
            results = pool.map( runCommand, data, chunksize = 1 )
        elif options.method == "drmaa":
            results = []
            runDRMAA( data, environment = options.environment )
        elif options.method == "threading":
            results = []
            def reportError( request, exc_info ):
                """report errors.
                """
                E.warn( "exception occured in request #%s: %s" % \
                            (request.requestID, exc_info[1]) )
                options.stdlog.flush()

            def saveResult( request, result ):
                """save result.
                """
                results.append( result )

            pool = threadpool.ThreadPool( options.cluster_num_jobs )
            for a in data:
                request = threadpool.WorkRequest( runCommand, 
                                                  args = (a,),
                                                  callback = saveResult,
                                                  exc_callback = reportError )

                pool.putRequest( request )
            pool.wait()
            
        niterations = 0
        for retcode, filename, cmd, logfile, iterations in results:
            niterations += iterations
            if not hasFinished( retcode, filename, options.output_tag, logfile ):
                failed_requests.append( (filename, cmd ))

    else:
        tmpdir = options.collect
        started_requests = [ (x[:-4], x) for x in glob.glob( tmpdir + "/*.out" ) ]

        E.info( "collecting %i files from %s" % (len(started_requests),
                                                 tmpdir ) )

    if failed_requests:
        for fn, cmd in failed_requests:
            E.error( "failed request: filename= %s, cmd= %s" % (fn,cmd) )
    else:
        E.info( "building result from %i parts" % len(started_requests) )

        if options.renumber:
            mapper = MapperLocal( pattern = options.renumber )
        else:
            mapper = MapperEmpty()

        ## deal with stdout
        name = None
        index = None

        for pattern, column in options.renumber_column:

            if re.search( pattern, "stdout"):
                try:
                    index = int(column) - 1
                except ValueError:
                    name = column
                    break

        if options.binary:
            ResultBuilderBinary()( started_requests, options.stdout, options )
        else:
            ResultBuilder( mapper = mapper, field_index = index, field_name = name )( started_requests, options.stdout, options )

        ## deal with logfiles : combine them into a single file
        x = re.search( "'--log=(\S+)'", cmd ) or re.search( "'--L\s+(\S+)'", cmd )
        if x:
            E.info( "logging output goes to %s" % x.groups()[0])
            logfile = IOtools.openFile(x.groups()[0], "a")
            ResultBuilderLog() ( [ (x[0], "%s.log" % x[0]) for x in started_requests ], logfile, options )
            logfile.close()

        ## deal with other files
        if options.subdirs:

            files = glob.glob( "%s/*.dir/*" % tmpdir )
            # remove directory
            filenames = set([ os.path.basename(x) for x in files ])
            xx = len(".out")

            for filename in filenames:

                _, filetype = os.path.splitext( filename )

                name = None
                index = None

                for pattern, column in options.renumber_column:
                    if re.search( pattern, filename):
                        try:
                            index = int(column) - 1
                        except ValueError:
                            name = column
                        break

                if options.binary:
                    builder = ResultBuilderBinary( mapper = mapper )
                elif filetype in ( ".fa", ".fasta" ):
                    builder = ResultBuilderFasta( mapper = mapper )
                elif filetype in ( ".mali", ):
                    builder = ResultBuilderFasta( mapper = MapperEmpty() )
                elif filetype in ( ".psl" ):
                    builder = ResultBuilderPSL( mapper = mapper )
                elif filetype in ( ".gtf", ".gff"):
                    builder = ResultBuilderGFF( mapper = mapper, field_index = index, field_name = name )
                elif filetype in ( ".png" ):
                    builder = ResultBuilderCopies( mapper = mapper )
                else:
                    builder = ResultBuilder( mapper = mapper, field_index = index, field_name = name )

                E.debug("chose the following builder for %s: %s: %s" % (filename, filetype, str(builder)))

                E.info( "collecting results for %s" % filename )

                input_filenames = []
                for fi, fn in started_requests:
                    fn = fn[:-xx] + ".dir/" + filename
                    if os.path.exists( fn ):
                        input_filenames.append( (fi, fn)  )

                E.info( "output of %i files goes to %s" % (len(filenames), filename ) )

                outfile = IOTools.openFile( options.output_pattern % filename, "w" )
                builder( input_filenames, outfile, options )
                outfile.close()

    if not options.debug and (not options.resume or not options.collect):
        if len(failed_requests) == 0:
            E.info("removing directory %s" % tmpdir )
            shutil.rmtree( tmpdir )
        else:
            E.info("directory %s not removed due to %i failed jobs" % (tmpdir, len(failed_requests) ) )
                
    E.info( "job control: nstarted=%i, nfinished=%i, nerrors=%i, nrepeats=%i" % \
                (len(started_requests),
                 len(started_requests) - len(failed_requests),
                 len(failed_requests),
                 niterations) )

    E.Stop()

if __name__ == '__main__':
    sys.exit(main())



import sys, os, re, collections, math, random
import numpy
import numpy.random

import CGAT.Stats as Stats
import CGAT.Experiment as E

from cnubiscan import *

def iterate_overlaps(matches):

    if len(matches) == 0: return
    #matches.sort( key = lambda x: (x.sequence, x.start) )
    matches.sort()
    lseq, lstart, lend = matches[0].sequence, matches[0].start, matches[0].end
    r = [matches[0]]
    for m in matches[1:]:
        if lseq != m.sequence or m.start > lend:
            yield (lseq, lstart, lend, r)
            lseq, lstart, lend = m.sequence, m.start, m.end
            r = []
        r.append( m )
        lend = max( m.end, lend )
    yield (lseq, lstart, lend, r) 

def combineMotifs( matches ):
    '''merge overlapping motifs.

    In the case of several overlapping motif arrangements, take the one with the
    highest zscore.
    '''
    filtered_results = []
    for seq, start, end, r in iterate_overlaps( matches ):
        r.sort( key = lambda x: x.zscore )
        if len(r) > 0:
            take = r[-1]._replace( alternatives = r[:-1])
        else:
            take = r[-1]
        filtered_results.append( take )
    return filtered_results

def runNubiscanAnalysis( sequences,
                        control_sequences,
                        arrangements,
                        sense_matrix ):

    matcher = Nubiscan( sense_matrix )
    
    all_sequences = sequences + control_sequences

    # find motifs in both foreground and control together
    # result = findNubiscanNuclearReceptors( all_sequences, arrangements )
    result = matcher.run( all_sequences,
                          arrangements,
                          zscore_threshold = 6.5 )
    
    nsequences = len(sequences)
    # separate foreground (fg) and control (background = bg)
    fg_result = [ x for x in result if x.sequence < nsequences ]
    bg_result = [ x for x in result if x.sequence >= nsequences ]

    fg_counter = E.Counter()
    bg_counter = E.Counter()
    fg_seqs = set()
    bg_seqs = set()
    
    for x in fg_result:
        fg_counter[x.arrangement] += 1
        fg_seqs.add( x.sequence )
    for x in bg_result:
        bg_counter[x.arrangement] += 1
        bg_seqs.add( x.sequence )

    print len(fg_result), len(bg_result) / 2
    print len(fg_seqs), len(bg_seqs) / 2

    for x in arrangements:
        print x, fg_counter[x], bg_counter[x] / 2

    fg_filtered = combineMotifs( fg_result )
    bg_filtered = combineMotifs( bg_result )
    
    fg_counter = E.Counter()
    bg_counter = E.Counter()
    fg_seqs = set()
    bg_seqs = set()
    
    for x in fg_filtered:
        fg_counter[x.arrangement] += 1
        fg_seqs.add( x.sequence )
    for x in bg_filtered:
        bg_counter[x.arrangement] += 1
        bg_seqs.add( x.sequence )

    print "###########################################"
    print len(fg_result), len(bg_result) / 2
    print len(fg_seqs), len(bg_seqs) / 2

    for x in arrangements:
        print x, fg_counter[x], bg_counter[x] / 2
    
    return []

def runAnalysis( sequences,
                 arrangements,
                 matrix = 'nr',
                 qvalue_threshold = 0.05 ):

    if matrix == 'nr':
        sense_matrix = NR
    elif matrix == "rxrvdr":
        sense_matrix = RXRVDR
    else:
        raise ValueError("unknown matrix")

    matcher = MatcherRandomisationSequence( sense_matrix )
    
    # find motifs in both foreground and control together
    results = []
    for x, sequence in enumerate(sequences):
        result = matcher.run( sequence,
                              arrangements,
                              qvalue_threshold = qvalue_threshold )
        
        for r in result:
            results.append( r._replace( sequence = x ) )
            
    nsequences = len(sequences)

    fg_filtered = combineMotifs( results )
    fg_counter = E.Counter()
    fg_seqs = set()
    co_counter = E.Counter()
    co_seqs = set()
    
    for x in results:
        fg_counter[x.arrangement] += 1
        fg_seqs.add( x.sequence )
        
    for x in fg_filtered:
        co_counter[x.arrangement] += 1
        co_seqs.add( x.sequence )

    for x in arrangements:
        print x, fg_counter[x], co_counter[x]

    print len(fg_seqs), len(co_seqs)

    return fg_filtered

if __name__ == "__main__":

    sequences = (
        'caaaggtcacagagttcattcatagacctgatgttcaagctctggctccagtaactgccatctgggtcgcttacccagcctgaatctcagtgttctcaac',
        'aaccatctgtgagactggttttgaacccagtgacctctgcctctgggggcagttctgtttccagtggttttgccacagcccagacagggttgaggttgct',
        'CTTCAAATTTGGTAGTAGTTATTTCTGTGAGGTCACTGGGTTCAGACCTTAACCTATCAACTGGAGTGTCTGCTCTAGAGCTAAGTGCTTTGCTGTGGAC',
        "ACATCCTGAACCCTGTGACCTAGGCCCAGGGCTGCTGCGCGGACGGTAGCTCCCCCTGCAGGAAGCAAGGTTCCTCCGGGCCCCCAGACTGCTGCTGGAC",
        "ATGAACTGTATGACCTCCTTTGAGGTCAGTATCCCTGCAGACAAAAATAGGGGCACCACCATTTACATGGCAGATCTGCCATGATGGTGAAAGCACCTGC")

    sequences = open("positive.fasta", "r" ).readlines()

    import random
    ss = []
    for x in sequences:
        xx = list(x)
        random.shuffle(xx)
        ss.append("".join(xx))
    sequences = ss

    sequences = ('NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNGTAAGTCATATCTCTGAACTTTAACTGTTGTTCCTCTAGCCAGCACCCCACCCCACCAGCTCCCATGTTTGTAAAAGCCTATTTTCCCCTTCCATTCCCTAGAGGGTTGGGTACTCAGGATAGGCCAGAAAGGGCAGGAAGAGAGAGACTCATCCTTACTAAAAGGAAAAGAAAGGGCAACAAGTTCAGCCATGCAGTTCATTTTCACTTTTGAATCCATTCTAAACATGACTAGCCATTATGTTAAAAGCAGGGGAGGTGAGGGGGGCAGGACTAGAAATTAGCATTTCTATATTTATGCTATACTTCAAATTAAACCAGTAATTTCAGTGCCAAAACTATGGAGAATAGGGCTGGGCATGGTGGCTTACACATGTAATCCCAGCATTTGGGGAGGTGGAAGCAGGCTGATC',)
    
    arrangements = [ "DR%s" % x for x in range(0,15) ] + [ "ER%s" % x for x in range(0,15) ]
    for r in runAnalysis( sequences, arrangements, matrix = 'rxrvdr' ):
        print str(r)




    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
table2bed.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Not generic. Takes a tab delimited table and converts to bed file

Usage
-----

Example::

   python table2bed.py --help

Type::

   python table2bed.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-t", "--table", dest="table", type="string",
                      help="supply input table name"  )
    parser.add_option("-o", "--outfile", dest = "outfile", type = "string",
                      help="supply output file name")

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    outfile = open(options.outfile, "w")
    for line in open(options.table).readlines():
        contig = line.split("\t")[0].split('"')[1]
        outfile.write("\t".join((contig, line.split("\t")[1], line.split("\t")[2])))
    
    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bam2bam.py - modify bam files
=============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a :term:`bam` formatted file from stdin and outputs a
modified :term:`bam` formatted file on stdout.

.. note::
   You need to redirect logging information to a file or turn it off
   via -v 0 in order to get a valid sam/bam file.

Usage
-----

Example::

   python bam2bam.py < in.bam > out.bam

Type::

   python bam2bam.py --help

for command line help.

Documentation
-------------

--set-nh: set the NH flag. Some tools (bowtie, bwa) do not set the NH flag. 
   If set, this option will set the NH flag (for mapped reads).
   This option requires the bam/sam file to be sorted by read name.

--unset-unmapped_mapq: some tools set the mapping quality of unmapped reads. This
   causes a violation in the Picard tools.

--remove-better: remove alignments that are worse than alignments in the :term:`bam`
   formatted file ``--filter-bam``. 

Code
----

'''

import os
import sys
import re
import optparse
import collections
import itertools

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam

try:
    import pyximport
    pyximport.install(build_in_temp=False)
    import _bam2bam
except ImportError:
    import CGAT._bam2bam as _bam2bam

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "--set-nh", dest="set_nh", action="store_true",
                       help = "sets the NH flag. The file needs to be sorted by readname [%default]" )

    parser.add_option( "--unset-unmapped-mapq", dest="unset_unmapped_mapq", action="store_true",
                       help = "sets the mapping quality of unmapped reads to 0 [%default]" )

    parser.add_option( "--strip", dest="strip", type="choice",
                       choices = ("sequence", "quality" ),
                       help = "remove parts of the bam-file. Note that stripping the sequence will "
                       "also strip the quality values [%default]" )

    parser.add_option( "--filter", dest="filter", action="append", type="choice",
                       choices=('NM', 'CM', 'mapped', 'unique' ),
                       help = "filter bam file. The option denotes the property that is  "
                       "used to determine better match [%default]" )

    parser.add_option( "--reference-bam", dest="reference_bam", type="string",
                       help = "bam-file to filter with [%default]" )
    
    parser.add_option( "--sam", dest="output_sam", action="store_true",
                       help = "output in sam format [%default]" )

    parser.set_defaults(
        filter = [],
        set_nh = False,
        unset_unmapped_mapq = False,
        output_sam = False,
        reference_bam = None,
        strip = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    # reading bam from stdin does not work with only "r" tag
    pysam_in = pysam.Samfile( "-", "rb" )

    if options.output_sam:
        pysam_out = pysam.Samfile( "-", "wh", template = pysam_in )
    else:
        pysam_out = pysam.Samfile( "-", "wb", template = pysam_in )

    if options.filter:

        if "NM" in options.filter:
            remove_mismatches = True
            colour_mismatches = False

        elif "CM" in options.filter:
            remove_mismatches = True
            colour_mismatches = True

        if remove_mismatches:
            if not options.reference_bam:
                raise ValueError( "requiring reference bam file for removing by mismatches")

            pysam_ref = pysam.Samfile( options.reference_bam, "rb" )
        else:
            pysam_ref = None

        c = _bam2bam.filter_bam( pysam_in, pysam_out, pysam_ref,
                                 remove_nonunique = "unique" in options.filter,
                                 remove_contigs = None,
                                 remove_unmapped = "mapped" in options.filter,
                                 remove_mismatches = remove_mismatches,
                                 colour_mismatches = colour_mismatches )
        
        options.stdlog.write( "category\tcounts\n%s\n" % c.asTable() )

    else:
        # set up the modifying iterators
        it = pysam_in.fetch( until_eof = True )

        if options.unset_unmapped_mapq:
            def unset_unmapped_mapq( i ):
                for read in i:
                    if read.is_unmapped:
                        read.mapq = 0
                    yield read
            it = unset_unmapped_mapq( it )

        if options.set_nh and False:
            def set_nh( i ):

                for key, reads in itertools.groupby( i, lambda x: x.qname ):
                    l = list(reads)
                    nh = len(l)
                    for read in l:
                        if not read.is_unmapped:
                            t = dict( read.tags )
                            t['NH'] = nh
                            read.tags = list(t.iteritems())
                        yield read
            it = set_nh( it )

        if options.strip != None:
            def strip_sequence( i ):
                for read in i:
                    read.seq = None
                    yield read

            def strip_quality( read ):
                for read in i:
                    read.qual = None
                    yield read

            if options.strip == "sequence":
                it = strip_sequence( it )
            elif options.strip == "quality":
                it = strip_quality( it )

        if options.set_nh:
            it = _bam2bam.SetNH( it )

        # read and output
        for read in it: 
            pysam_out.write( read )

        pysam_in.close()
        pysam_out.close()

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat_html_add_toc.py - insert table of contents in html document
================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Read an html document on stdin and add a table of contents
based on section headings in the document.

This document uses the ``<h1></h1>``, ``<h2></h2>``, ... html tags.

Usage
-----

Example::

   python cgat_html_add_toc.py --help

Type::

   python cgat_html_add_toc.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import re
import sys
import time
import getopt
import StringIO
from types import *

def usage():
    print """Usage: %s [--max-depth N] [--no-toc-links] file.html file-created.html
--max-depth N: The depth of the table of content should be N.
--no-toc-links: Don't insert links to the TOC after each heading.
--hide:
  Hide all headings and the content "below" which is marked with class="hide"
  Example: <h2 class="hide">Ignore Me</h2>
  
Number headings in a HTML file. The headings get numbered by
the <h?> tags in your file. You can insert a TOC (Table of Content)
with <!-- TOC -->.

  Example:
  <h1>First</h1>
  <h2>Sub-Item</h2>
  <h2>Sub-Item</h2>

  <h1>Second</h1>

  Result:

  <h1>1 First</h1>
  <h2>1.1 Sub-Item</h2>
  <h2>1.2 Sub-Item</h2>

  <h1>2 Second</h1>    """ % (
        os.path.basename(sys.argv[0]))

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "", ["max-depth=",
                                                      "no-toc-links",
                                                      "hide",
                                                      "help",
                                                      "version"])
    except getopt.GetoptError, msg:
        usage()
        print msg
        sys.exit(1)

    max_depth=3
    no_toc_links=0
    do_hide=False
    for o, a in opts:
        if o == "--help":
            usage()
            sys.exit(0)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o=="--no-toc-links":
            no_toc_links=1
        elif o=="--max-depth":
            try:
                max_depth=int(a)
            except ValueError:
                sys.stderr.write("%s must be a number\n" % a)
                sys.exit(1)
        elif o=="--hide":
            do_hide=True
        else:
            raise("Internal Error %s %s not processed" % (
                o, a))

    if len(args)!=2:
        usage()
        sys.exit(1)

    filename_in=args[0]
    filename_out=args[1]
    fd=open(filename_in)
    file=fd.readlines()
    fd.close()

    re_heading=re.compile(r'^(.*)<h(\d)([^>]*)>(.*?)<(/)?h(\d)>(.*)$')

    stack=[]
    last_level=0
    out=StringIO.StringIO()
    out_orig=out
    hide_out=StringIO.StringIO()
    hide=None
    headings=[]
    first = True
    for line in file:
        match=re_heading.match(line)
        if not match:
            out.write(line)
        else:
            heading=match.group(4)
            before_heading=match.group(1)
            attributes=match.group(3)
            
            after_heading=match.group(7)
            if match.group(2)!=match.group(6):
                raise "Parse Error: <h?> does not match </h?> %s %s: '%s' " % (
                    match.group(2), match.group(6), line)
            if match.group(5)!="/":
                raise "Parse Error: Missing slash in end-tag: '%s'" % line.strip()
            level=int(match.group(2))
            assert(level>0 and level<10)
            ## deal with TOCs, that do not start with <h1>
            if first:
                last_level = level - 1
                first = False
            if level==last_level+1:
                stack.append(1)
            elif level==last_level:
                stack[-1]+=1
            elif level<last_level:
                diff=last_level-level
                if diff>len(stack):
                    raise("Parse Error: diff: %s stack: %s last_level: %s"
                          " level: %s" % (
                        diff, stack, last_level, level))
                for i in range(diff):
                    stack.pop()
                stack[-1]+=1
            else:
                raise("Strange sequence of <h?> Stack: %s "
                      " level:%s last_level:%s line: '%s'" %
                      (stack, level, last_level, line))
            if hide and level<=hide:
                out=out_orig
                hide=None
            last_level=level
            number=[]
            space=[]
            number_td=[]
            for i in range(len(stack)):
                number.append(str(stack[i]))
                if i!=0:
                    space.append('&nbsp;&nbsp;')
            space=''.join(space)
            number='.'.join(number)

            match=re.match(r'^(.*?)\s*id="(.+?)"\s*(.*)$', attributes)
            if match:
                # If the heading has already an id, then take this name
                # and don't use link_N
                id=match.group(2)
                attributes="%s %s" % (match.group(1), match.group(3))
            else:
                id="link_%s" % number

            if do_hide:
                match=re.match(r'^(.*?)\s*class="hide"\s*(.*)$', attributes)
                if match:
                    # hide this heading and all stuff below
                    assert hide==None
                    hide=level
                    attributes="%s %s" % (match.group(1), match.group(2))
                    out=hide_out

            # Create column for each number in heading (for alignment)
            #print stack, heading, number
            if len(stack)==1:
                size="font-size: 100%"
            elif len(stack)==2:
                size="font-size: 80%"
            else:
                size="font-size: 60%"
            for i in range(max_depth):
                if i==len(stack)-1:
                    point="&nbsp;"
                    value=stack[i]
                elif i<len(stack):
                    point="."
                    value=stack[i]
                else:
                    point="&nbsp;"
                    value=""
                number_td.append(
                    '<td align="right" style="%s">%s%s</td>' % (
                    size, value, point))
            number_td=''.join(number_td)
            
            if no_toc_links:
                toc=""
            else:
                toc='<a style="font-size: 50%%" href="#toc_%s">[toc]</a>' % (
                    number)
                
            out.write(before_heading)
            out_orig.write('''
            <h%s %s id="%s">%s %s
             %s
            </h%s>''' % (level, attributes,
                   id, number, heading, toc,
                   level))
            if hide==level:
                out_orig.write("...")
            else:
                out.write(after_heading)
            assert(type(level)==IntType)

            #  create heading
            if level<=max_depth:
                headings.append('''
                 <tr>
                  %s
                  <td><a name="toc_%s" href="#%s">%s%s</a></td>
                 </tr>''' % (number_td, number,
                             id, space, heading))
                             
                
    headings='''
     <table>
      %s
     </table>''' % ''.join(headings)
    out=out.getvalue()
    out=re.sub(r'<!--\s*TOC\s*-->', ''.join(headings), out)
    out=re.sub(r'<html>',
               '''<html>
    <!--
     DO NOT EDIT!
    created by number-html-headings.py on %s from %s
     DO NOT EDIT!
     -->''' %
               (time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), filename_in),
               out)
    out=re.sub(r'\s*</pre>', r'</pre>', out, re.DOTALL)
    fd=open(filename_out, "wt")
    fd.write(out)
    fd.close()

if __name__=="__main__":
    main()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Logfile.py - logfile parsing
============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Parse logfiles

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import collections, re

RuntimeInformation = collections.namedtuple( "RuntimeInformation",
                                             "script options jobid host has_finished " +\
                                             "start_date end_date " +\
                                             "wall utime stime cutime cstime" )

RX_START = re.compile( "output generated by (\S+) (.*)" )
RX_JOB = re.compile( "job started at (.*) on (\S+) -- (\S+)" )
RX_FINISH = re.compile( "job finished in (\S+) seconds at (.*) --\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+) -- (\S+)")

def parse( filename ):
    
    results = []
    script, options, started, finished, host, jobid = "", "", "?", "?", "?", "?"
    wall, utime, stime, cutime, cstime = [0] * 5
    with open(filename) as inf:
        for line in inf:
            x = RX_START.search( line )
            if x:
                if script:
                    results.append( RuntimeInformation._make( (\
                                script, options, 
                                jobid, host,
                                finished != "",
                                started, finished, 
                                int(wall), float(utime), float(stime), float(cutime), float(cstime)) ) )
                script, options, started, finished, host, jobid = "", "", "?", "?", "?", ""
                wall, utime, stime, cutime, cstime = [0] * 5
                script, options = x.groups()
                continue
            x = RX_JOB.search(line)
            if x:
                started, host, jobid = x.groups() 
                continue
            x = RX_FINISH.search(line)
            if x:
                wall, finished, utime, stime, cutime, cstime, _jobid = x.groups()
                assert _jobid == jobid

    results.append( RuntimeInformation._make( (\
            script, options, 
            jobid, host,
            finished != "",
            started, finished, 
            int(wall), float(utime), float(stime), float(cutime), float(cstime)) ) )
            
    return results


class LogFileData:
    mRegex = re.compile("# job finished in (\d+) seconds at (.*) --\s+([.\d]+)\s+([.\d]+)\s+([.\d]+)\s+([.\d]+)")
    mFormat = "%6.2f"
    mDivider = 1.0
    
    def __init__(self):

        self.mWall = 0
        self.mUser = 0
        self.mSys = 0
        self.mChildUser = 0
        self.mChildSys = 0
        self.mNChunks = 0

    def add( self, line ):

        if not self.mRegex.match(line): return
        t_wall, date, t_user, t_sys, t_child_user, t_child_sys = self.mRegex.match(line).groups()

        self.mWall += int(t_wall)
        self.mUser += float(t_user)
        self.mSys += float(t_sys)
        self.mChildUser += float(t_child_user)
        self.mChildSys += float(t_child_sys)
        self.mNChunks += 1

    def __getitem__( self, key ):
        
        if key == "wall":
            return self.mWall
        elif key == "user":
            return self.mUser
        elif key == "sys":
            return self.mSys
        elif key == "cuser":
            return self.mChildUser
        elif key == "csys":
            return self.mChildSys
        elif key == "nchunks":
            return self.mNChunks
        else:
            raise ValueError("key %s not found" % key)

    def __add__(self, other ):

        self.mWall += other.mWall
        self.mUser += other.mUser
        self.mSys += other.mSys
        self.mChildUser += other.mChildUser
        self.mChildSys += other.mChildSys
        self.mNChunks += other.mNChunks

        return self
    
    def __str__(self):
        
        return "%i\t%s" % (
            self.mNChunks,
            "\t".join( map( lambda x: self.mFormat % (float(x)/self.mDivider), \
                                (self.mWall, self.mUser, self.mSys,
                                 self.mChildUser, self.mChildSys ) ) ) )

    def getHeader(self):
        return "\t".join( ("chunks", "wall", "user", "sys", "cuser", "csys") )


class LogFileDataLines(LogFileData):
    """record lines."""
    def __init__(self):
        LogFileData.__init__(self)
        self.mNLines = 0

    def add( self, line ):
        if line[0] != "#":
            self.mNLines += 1
        else:
            return LogFileData.add( self, line )
    def __getitem__( self, key ):
        if key == "lines":
            return self.mNLines
        else:
            return LogFileData.__getitem__( self, key )
    def __add__(self, other ):
        self.mNLines += other.mNLines
        return LogFileData.__add__( self, other )

    def __str__(self):
        return "%s\t%i" % (LogFileData.__str__(self), self.mNLines )
                          
    def getHeader(self):
        return "%s\t%s" % (LogFileData.getHeader(self), "lines" )
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2013 Stephen Sansom
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
run_function.py - run a function within a python module remotely
=================================================================

:Author: Stephen Sansom, with credit to Tim for implementation suggestions.
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script allows a specificied function from a specified python module 
to be executed on the cluster with specified parameters.

Usage
-----

A statement is specified in the normal way i.e.::

   statement = """python %(scriptsdir)s/run_function.py 
                   -p infile,outfile,additional_param1
                   -m modulefile 
                   -f function"""

   P.run()

If the module is in your $PYTHONPATH you can just name it 
directly. i.e "Pipeline" would specifiy Pipeline.py

The script has currently only been tested with single input/output.

Documentation
-------------

Code
----

'''

import sys
import os
import importlib

import CGAT.Experiment as E

def main(argv = None):
    
    # Parse the options
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $",
                             usage = globals()["__doc__"] )

    parser.add_option("-p", "--params", dest="params", type="string",
                      help="comma separated list of addtional parameter strings")

    parser.add_option("-m", "--module", dest="module", type="string",
                      help="the full path to the module file", default = None)

    parser.add_option("-i", "--input", dest="input_filenames", type="string", action="append",
                      help="input filename")

    parser.add_option("-o", "--output", dest="output_filenames", type="string", action="append",
                      help="output filename")

    parser.add_option("-f", "--function", dest="function", type="string",
                      help="the module function", default = None)
    
    parser.set_defaults( 
        input_filenames = [],
        output_filenames = [],
        params = None
        )

    (options, args) = E.Start( parser )

    # Check a module and function have been specified
    if not options.module or not options.function:
        raise ValueError("Both a function and Module must be specified")

    # If a full path was given, add this path to the system path
    location = os.path.dirname(options.module)
    if location != "":
        sys.path.append(location)
   
    # Establish the module name, accomodating cases where the
    # .py extension has been included in the module name
    module_name = os.path.basename(options.module)
    if module_name.endswith(".py"):
        module_base_name = module_name[:-3]
    else:
        module_base_name = module_name

    # Import the specified module and map the specified fuction
    E.info( "importing module '%s' " % module_base_name)
    E.debug( "sys.path is: %s" % sys.path )

    module = importlib.import_module( module_base_name )
    try:
        function = getattr(module, options.function)
    except AttributeError as msg:
        raise AttributeError( msg.message + "unknown function, available functions are: %s" % \
                                  ",".join( [ x for x in dir(module) if not x.startswith("_") ] ) )

    if options.input_filenames:
        infiles = options.input_filenames
    else:
        infiles = False

    if options.output_filenames:
        outfiles = options.output_filenames
    else:
        outfiles = False

    # Parse the parameters into an array
    if options.params:
        params = [param.strip() for param in options.params.split(",")]
    else:
        params = False
              
    # deal with single file case
    if len(infiles) == 1: infiles = infiles[0]
    if len(outfiles) == 1: outfiles = outfiles[0]

    # Make the function call
    if infiles and outfiles and params:
        function(infiles, outfiles, params)
    elif infiles and outfiles and not params:
        function(infiles, outfiles)
    elif params:
        function(params)
    else:
        raise ValueError("Expecting infile+outfile+params or infile+outfile or params")

if __name__ == "__main__":
    sys.exit(main(sys.argv))



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: gtf2fasta.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
gtf2fasta.py - annotate genomic bases from a gene set
=====================================================

:Author: Andreas Heger
:Release: $Id: gtf2fasta.py 2861 2010-02-23 17:36:32Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

This script takes a :term:`gtf` formatted file from ENSEMBL and annotates each base 
in the genome according to its *function*. The script multiplexes both strands with lower-
case characters referring to the forward strand and upper-case characters
referring to the reverse strand.

The codes and their meaning are:

+---------------+----------------------------------------------------------------------+                                                                                                                                                                   
|code           | description                                                          |
+---------------+----------------------------------------------------------------------+                                                                                                                                                                   
|a              | first codon position within a complete codon                         |                                                                                                                                                                   
+---------------+----------------------------------------------------------------------+                                                                                                                                                                   
|b              | second codon position within a complete codon                        |
+---------------+----------------------------------------------------------------------+
|c              | third codon position within a complete codon                         |
+---------------+----------------------------------------------------------------------+
|d              | coding base, but in multiple frames or strands                       |
+---------------+----------------------------------------------------------------------+
|e              | non-coding base in exon                                              |
+---------------+----------------------------------------------------------------------+
|f              | frame-shifted base                                                   |
+---------------+----------------------------------------------------------------------+
|g              | intergenic base                                                      |
+---------------+----------------------------------------------------------------------+
|i              | intronic base                                                        |
+---------------+----------------------------------------------------------------------+
|l              | base in other RNA                                                    |
+---------------+----------------------------------------------------------------------+
|m              | base in miRNA                                                        |
+---------------+----------------------------------------------------------------------+
|n              | base in snRNA                                                        |
+---------------+----------------------------------------------------------------------+
|o              | base in snoRNA                                                       |
+---------------+----------------------------------------------------------------------+
|r              | base in rRNA (both genomic and mitochondrial)                        |
+---------------+----------------------------------------------------------------------+
|p              | base in pseudogene (including transcribed, unprocessed and processed)|
+---------------+----------------------------------------------------------------------+
|q              | base in retrotransposon                                              |
+---------------+----------------------------------------------------------------------+
|s              | base within a splice signal (GT/AG)                                  |
+---------------+----------------------------------------------------------------------+
|t              | base in tRNA (both genomic and mitochondrial)                        |
+---------------+----------------------------------------------------------------------+
|u              | base in 5' UTR                                                       |
+---------------+----------------------------------------------------------------------+
|v              | base in 3' UTR                                                       |
+---------------+----------------------------------------------------------------------+
|x              | ambiguous base with multiple functions.                              |
+---------------+----------------------------------------------------------------------+
|y              | unknown base                                                         |
+---------------+----------------------------------------------------------------------+

This script can be used for a quick-and-dirty annotation of variants in a genome.
For a better prediction of variant effects in coding variants, see :doc:`gtf2alleles`.

Output files
++++++++++++

The annotated genome is output on stdout.

The script creates the following additional output files:

counts
   Counts for each annotations

junctions
   Splice junctions. This is a tab separated table linking residues that are
   joined via features. The coordinates are forward/reverse coordinates.

   The columns are:

   contig
      the contig
   strand
      direction of linkage
   end
      last base of exon in direction of strand
   start
      first base of exon in direction of strand
   frame
      frame base at second coordinate (for coding sequences)
    
Known problems
--------------

The stop-codon is part of the UTR. This has the following effects:

   * On the mitochondrial chromosome, the
     stop-codon might be used for ncRNA transcripts and thus the base is
     recorded as ambiguous. 

   * On the mitochondrial chromosome, alternative transcripts might read
     through a stop-codon (RNA editing). The codon itself will be recorded
     as ambiguous.

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse
import string
import collections
import array

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Genomics as Genomics
import CGAT.Intervals as Intervals

# mapping ensembl annotations in the source field
MAP_ENSEMBL = { 'miRNA' : 'm',
                'misc_RNA' : 'l',
                'pseudogene' : 'p',
                'transcribed_pseudogene' : 'p',
                'unprocessed_pseudogene' : 'p',
                'processed_pseudogene' : 'p',
                'retrotransposed' : 'q',
                'rRNA' : 'r',
                'snRNA' : 'n',
                'snoRNA' : 'o',
                'Mt_rRNA' : 'r',
                'Mt_tRNA' : 't' }

DEFAULT_CODE = "g"
AMBIGUOUS_CODE = "x"
CODING_CODE = "d"
ALL_CODES = DEFAULT_CODE + AMBIGUOUS_CODE + "abcdefilmnorpqstuvy" + "abcdefilmnorpqstuvy".upper()
CODING_CODES = "abcdABCD"
NONCODING_CODES = "iuvIUV"
INTRON_CODES = "iI"
UTR_CODES = "uvUV"

def setCode( annotation, pos, code ):
    """set *pos* to *code* in annotation.
 
    This method performs conflict resolution in the following cases:

    1. Introns and UTRs do not cause a conflict if they overlap with other
    features

    1. Coding bases take precedence over intronic and UTR bases.
    2. UTR base take precedence over intronic bases.
    3. Coding bases in different frames/strands are set to CODING_CODE
    4. Non-coding features of the same type but different strand are permitted
    
    All other conflicts are marked as ambiguous bases.
    """
    c = annotation[pos]
    if c == DEFAULT_CODE or c in NONCODING_CODES:
        annotation[pos] = code
    elif c == AMBIGUOUS_CODE:
        return
    elif code in NONCODING_CODES:
        # only set introns/UTR if no other code is present
        return
    elif c == code:
        return
    elif code in CODING_CODES and c in CODING_CODES:
        # ambiguous frame/strand in coding sequence
        annotation[pos] = CODING_CODE
    elif code in NONCODING_CODES and c in CODING_CODES:
        # permit alternative transcripts
        return
    elif c not in CODING_CODES and code not in CODING_CODES and c.upper() == code.upper():
        # permit features of the same type on different strands to overlap (for example, tRNAs)
        return
    else:
        annotation[pos] = AMBIGUOUS_CODE
        E.warn( "ambiguous position %i: %s - %s" % (pos, c, code) )

#     elif code in CODING_CODES and c in NONCODING_CODES:
#         # permit alternative transcripts
#         annotation[pos] = code
#     elif code in INTRON_CODES and c in UTR_CODES:
#         # permit alternative transcripts in non-coding parts of protein coding genes
#         # giving preference to UTR
#         return
#     elif code in UTR_CODES and c in INTRON_CODES:
#         # permit alternative transcripts in non-coding parts of protein coding genes
#         # giving preference to UTR
#         annotation[pos] = code

def addSegments( annotation, intervals, is_positive, code ):
    """add intervals."""
    if not intervals : return 
    if not is_positive: code = code.upper()

    for start,end in intervals:
        for x in xrange( start, end):
            setCode( annotation, x, code )

def addIntrons( annotation, intervals, is_positive, max_frameshift_length ):
    """add introns for intervals.

    Intervals need to be sorted in incremental order.
    """
    if not intervals : return 
    intervals.sort()
    last = intervals[0][1]
    code_i, code_f, code_s = "i", "f", "s"
    
    if not is_positive: 
        code_i = code_i.upper()
        code_f = code_f.upper()
        code_s = code_s.upper()

    for start,end in intervals[1:]:
        d = start - last
        if d < max_frameshift_length:
            code = code_f
        else:
            code = code_i
            # add splice sites
            setCode( annotation, last, code_s)
            setCode( annotation, last+1, code_s)
            setCode( annotation, start-2, code_s)
            setCode( annotation, start-1, code_s)
            last += 2
            start -= 2
            
        for x in xrange( last, start ):
            setCode( annotation, x, code )

        last = end

def addCDS( annotation, gtfs, is_positive ):
    """add coding sequence to contig.
    
    Also adds the splice sites.
    """

    if not gtfs: return

    if is_positive:
        chars = "abc"
    else:
        chars = "ABC"

    last = None
    for cds in gtfs:

        c = int(cds.frame)
        if c != 0: 
            c = 3 - c

        if is_positive:
            r = xrange(cds.start, cds.end)
        else:
            r = xrange(cds.end-1, cds.start-1,-1)

        for x in r:
            code = chars[c]
            c += 1 
            if c == len(chars): c = 0
            setCode( annotation, x, code )

def outputCounts( outfile, annotations ):
    """output table into outfile with annotations."""

    total_counts = collections.defaultdict( int )        
    
    outfile.write( "contig\ttotal\t%s\n" % ("\t".join(ALL_CODES) ) )

    for k in sorted(annotations.keys()):
        counts = collections.defaultdict( int )
        for x in annotations[k]: counts[x] += 1
        outfile.write( "\t".join( (k, 
                                  str(len(annotations[k])),
                                  "\t".join( [str(counts[x]) for x in ALL_CODES]) ) ) + "\n" )

        for k,v in counts.iteritems(): total_counts[k] += v
        
    outfile.write( "\t".join( ("total", 
                               str(sum( [len(x) for x in annotations.values() ] )),
                               "\t".join( [str(total_counts[x]) for x in ALL_CODES]) ) ) + "\n" )
        

def annotateGenome( iterator, fasta, options, default_code = DEFAULT_CODE ):
    """annotate a genome given by the indexed *fasta* file and 
    an iterator over gtf annotations.
    """
    
    annotations = {}
    contig_sizes = fasta.getContigSizes( with_synonyms = False )
    E.info("allocating memory for %i contigs and %i bytes" % \
               (len(contig_sizes), sum(contig_sizes.values()) * array.array("c").itemsize) )
          # AString.AString( "a").itemsize ))

    for contig,size in contig_sizes.items():
        E.debug( "allocating %s: %i bases" % (contig, size ) )
        # annotations[contig] = AString.AString( default_code * size )
        annotations[contig] = array.array( "c", default_code * size )

    E.info("allocated memory for %i contigs" % len(fasta))

    counter = E.Counter()

    # output splice junctions
    outfile_junctions = E.openOutputFile( "junctions" )
    outfile_junctions.write( "contig\tstrand\tpos1\tpos2\tframe\tgene_id\ttranscript_id\n" )
    for gtfs in iterator:

        counter.input += 1

        if counter.input % options.report_step == 0:
            E.info( "iteration %i" % counter.input )

        try:
            contig = fasta.getToken( gtfs[0].contig )
        except KeyError, msg:
            E.warn( "contig %s not found - annotation ignored" % gtfs[0].contig )
            counter.skipped_contig += 1
            continue

        lcontig = fasta.getLength( contig )

        # make sure that exons are sorted by coordinate
        gtfs.sort( key = lambda x: x.start )

        is_positive = Genomics.IsPositiveStrand( gtfs[0].strand )
        source = gtfs[0].source 

        # process non-coding data
        if source in MAP_ENSEMBL:
            code = MAP_ENSEMBL[ source ]

            intervals = [(x.start, x.end) for x in gtfs]
            addSegments( annotations[contig], 
                         intervals,
                         is_positive,
                         code )

        elif source == "protein_coding":
            
            # collect exons for utr
            exons = [ (x.start, x.end) for x in gtfs if x.feature == "exon" ]
            cds = [ (x.start, x.end) for x in gtfs if x.feature == "CDS" ]
            if len(cds) == 0:
                counter.skipped_transcripts += 1
                E.warn( "protein-coding transcript %s without CDS - skipped" % gtfs[0].transcript_id )
                continue

            exons = Intervals.truncate( exons, cds )
            start, end = cds[0][0], cds[-1][1]

            UTR5 = [ x for x in exons if x[1] < start ]
            UTR3 = [ x for x in exons if x[0] >= end ]

            if not is_positive:
                UTR5, UTR3 = UTR3, UTR5
                splice_code = "S"
            else:
                splice_code = "s"

            addSegments( annotations[contig], 
                         UTR5,
                         is_positive,
                         "u" )

            addIntrons( annotations[contig], 
                        UTR5, 
                        is_positive,
                        options.max_frameshift_length )

            addSegments( annotations[contig], 
                         UTR3,
                         is_positive,
                         "v" )

            addIntrons( annotations[contig], 
                        UTR3, 
                        is_positive,
                        options.max_frameshift_length )

            # output CDS according to frame
            addCDS( annotations[contig],
                    [ x for x in gtfs if x.feature == "CDS" ],
                    is_positive )

            # add introns between CDS
            addIntrons( annotations[contig], 
                        cds, 
                        is_positive,
                        options.max_frameshift_length )

            # output splice junctions
            cds = [ x for x in gtfs if x.feature == "CDS" ]
            
            # apply corrections for 1-past end coordinates
            # to point between residues within CDS
            if is_positive:
                ender = lambda x: x.end - 1
                starter = lambda x: x.start
                out_positive = "+"
            else:
                ender = lambda x: lcontig - x.start - 1
                starter = lambda x: lcontig - x.end 
                out_positive = "-"
                cds.reverse()

            end = ender( cds[0] )
            for c in cds[1:]:
                start = starter(c)
                outfile_junctions.write( "%s\t%s\t%i\t%i\t%s\t%s\t%s\n" %\
                                             ( contig,
                                               out_positive,
                                               end, 
                                               start,
                                               c.frame,
                                               c.gene_id,
                                               c.transcript_id,
                                               ) )
                end = ender(c)

    E.info( "finished reading genes: %s" % str(counter) )

    outfile_junctions.close()

    E.info( "started counting" )
    outfile = E.openOutputFile( "counts" )
    outputCounts( outfile, annotations )
    outfile.close()

    E.info( "started output" )
    for k in sorted(annotations.keys()):
        options.stdout.write( ">%s\n%s\n" % (k,annotations[k].tostring()) )

    
def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: gtf2fasta.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.add_option("-i", "--ignore-missing", dest="ignore_missing", action="store_true",
                      help="Ignore transcripts on contigs that are not in the genome-file [default=%default]."  )

    parser.add_option( "--min-intron-length", dest="min_intron_length", type="int",
                      help="minimum intron length. If the distance between two consecutive exons is smaller, the region will be marked 'unknown' [default=%default]."  )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices=("full", ),
                      help="method to apply [default=%default]."  )

    parser.set_defaults(
        genome_file = None,
        flank = 1000,
        max_frameshift_length = 4,
        min_intron_length = 30,
        ignore_missing = False,
        restrict_source = None,
        method = "full",
        report_step = 1000,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    if not options.genome_file:
        raise ValueError( "an indexed genome is required." )

    fasta = IndexedFasta.IndexedFasta( options.genome_file )

    iterator = GTF.transcript_iterator( GTF.iterator( options.stdin) )

    annotateGenome( iterator, fasta, options )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
tree_map_leaves.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python tree_map_leaves.py --help

Type::

   python tree_map_leaves.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import popen2

from Bio.Nexus import Nexus
from Bio.Nexus.Nodes import Node


USAGE="""python %s [OPTIONS] < tree.in > tree.out

Version: $Id: tree_map_leaves.py 2782 2009-09-10 11:40:29Z andreas $

collapse leafs with the same species.

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-a, --apply=                    apply mapping in file
-c, --create=                   create mapping in file
-i, --invert=                   invert map
-b, --strip-branches            remove branch lengths
""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools

param_loglevel = 1

param_long_options=["verbose=", "help", "pattern-species=",
                    "invert", "create=", "apply=", "strip-brances",
                    "version"]

param_short_options="v:hp:a:c:i"

param_apply = None
param_create = None
param_invert = False
param_remove_branch_lengths = False

if __name__ == "__main__":

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)
        elif o in ("-a", "--apply"):
            param_apply = a
        elif o in ("-c", "--create"):
            param_create = a
        elif o in ("-i", "--invert"):
            param_invert = True
        elif o in ("-b", "--strip-branches"):
            param_remove_branch_lengths = True

    print E.GetHeader()
    print E.GetParams()

    keys = {}
    if param_apply:
        infile = open(param_apply, "r")
        for line in infile:
            if line[0] == "#": continue
            a, b = line[:-1].split("\t")[:2]
            if param_invert:            
                a, b = b, a
            keys[a] = b

    nexus = TreeTools.Newick2Nexus( sys.stdin )

    notu = 0
    
    for tree in nexus.trees:
        if param_loglevel >= 2:
            tree.display()

        for nx in tree.get_terminals():
            t1 = tree.node(nx).get_data().taxon
            
            if param_create:
                if t1 not in keys:
                    keys[t1] = "otu%i" % notu
                    notu += 1

            if t1 in keys:
                tree.node(nx).get_data().taxon = keys[t1]

    print TreeTools.Nexus2Newick( nexus )

    if param_create:
        outfile = open(param_create, "w")
        for key in keys:
            outfile.write("%s\t%s\n" % (key, keys[key]))
        outfile.close()

    print E.GetFooter()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fastqs2fasta.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script is used to interleave two :term:`fastq`-formatted files (paired data) into a single :term:`fasta`-formatted file. Read1 is followed by 
read2 in the resultant file. 

:term:`fastq` files MUST be sorted by read identifier.

Usage
-----

Example::

   python fastqs2fasta.py --fastq1 in.fastq.1.gz --fastq2 in.fastq.2.gz > out.fasta

Type::

   python fastqs2fasta.py --help

for command line help.


Code
----

'''

import os
import sys
import re
import optparse
import itertools
import CGAT.IOTools as IOTools
import CGAT.Fastq as Fastq

import CGAT.Experiment as E

class PairedReadError(Exception):
    '''
    exception raised when reads aren't paired -
    could be not sorted or files of different lengths
    '''
    pass

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-a", "--fastq1", dest="fastq1", type="string",
                      help="supply read1 fastq file"  )
    parser.add_option("-b", "--fastq2", dest="fastq2", type="string",
                      help="supply read2 fastq file"  )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    fastq1 = IOTools.openFile(options.fastq1)
    fastq2 = IOTools.openFile(options.fastq2)

    E.info("iterating over fastq files")
    f1_count = 0
    for f1, f2 in itertools.izip_longest(Fastq.iterate(fastq1), Fastq.iterate(fastq2)):
        if not (f1 and f2) or (not f2 and f1):
            try:
                raise PairedReadError("unpaired reads detected. Are files sorted? are files of equal length?")
            except PairedReadError, e:
                raise PairedReadError(e), None, sys.exc_info()[2]
        else:
            assert f1.identifier.endswith("/1") and f2.identifier.endswith("/2"), "Reads in file 1 must end with /1 and reads in file 2 with /2"
            options.stdout.write(">%s\n%s\n>%s\n%s\n" % (f1.identifier, f1.seq, f2.identifier, f2.seq))
            f1_count += 1

    E.info("output: %i pairs" % f1_count)

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
map_residues.py - liftover for residues
=======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

maps a list of residues in a master sequence based on an explicit
(multiple) alignment.

Options::

    -m, --master    number of sequence to use as master (default is first row)
    -f, --format    alignment format (plain is the default and so far the only
                    supported option)       

Usage
-----

Example::

   python map_residues.py --help

Type::

   python map_residues.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import re
import os
import string
import getopt
import alignlib

param_master = 0
param_format = None
param_multiple_alignment = None

GAPCHARS=(".", "-")

if __name__ == '__main__':
     
    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                                      "V:m:f:h",
                                      ["Verbose=", "master=", 
                                       "format=", "help", "version"])
                                       
    except getopt.error, msg:
        print globals()["__doc__"], msg
        sys.exit(2)

    for o,a in optlist:
        if o in ("-h", "--help"):
            print globals()["__doc__"]
            sys.exit(0)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ("-f", "--format"):
            param_format = a
        elif o in ("-m", "--master"):
            param_master = string.atoi(a)

    if len(args) != 1:
        print "please specify a multiple alignment."
        print globals()["__doc__"]
        sys.exit(1)

    lines = map( lambda x: string.split(x[:-1], "\t"), open(args[0], "r").readlines())

    residues = filter( lambda x: not re.match("#", x), sys.stdin.readlines())

    if len(lines) <= param_master:
        print "master alignment does no exist in the alignment."
        print globals()["__doc__"]
        sys.exit(1)

    master_from, master_ali, master_to, master_id = lines[param_master]

    print "# master=", master_id

    for index in range(len(lines)):
        if param_master == index: continue

        sbjct_from, sbjct_ali, sbjct_to, sbjct_id = lines[index]

        print "#", sbjct_id
        map_master2sbjct = alignlib.makeAlignataVector()

        sbjct_index  = string.atoi(sbjct_from)
        master_index = string.atoi(master_from)
        
        map_sbjct2residue = {}
        
        for x in range(len(master_ali)):
            
            if master_ali[x] not in GAPCHARS and sbjct_ali[x] not in GAPCHARS:
                map_master2sbjct.addPairExplicit( master_index, sbjct_index, 0)
                map_sbjct2residue[sbjct_index] = sbjct_ali[x]

            if master_ali[x] not in GAPCHARS:                
                master_index +=1

            if sbjct_ali[x] not in GAPCHARS:
                sbjct_index += 1

        for line in residues:
            r = string.atoi(string.split(line[:-1], "\t")[0])
            sbjct_index = map_master2sbjct.mapRowToCol(r)
            if sbjct_index:
                print string.join( (str(sbjct_index), map_sbjct2residue[sbjct_index]), "\t") + "\t" + line[:-1]
            else:
                print string.join( ("0", "X"), "\t") + "\t" + line[:-1]
                








################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Masker.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, subprocess, tempfile, string, re, random

import Experiment as E
import Genomics, FastaIterator
import cStringIO as StringIO

## Class for calling masking programs.
class Masker :
    """a masker preserves gaps, but it does not preserve
    whitespace characters."""
    
    mExecutable = None
    mOptions=""
    mHasPeptideMasking = False
    mHasNucleicAcidMasking = False

    def __init__(self):
        pass

    def getAlphabet( self, sequence ):
        """get sequence type (aa,na,codons)."""

        s1 = re.sub( "[acgtxn\-.]", "", sequence.lower())
        s2 = re.sub( "[-.]", "", sequence.lower())
        if float(len(s1)) < (len(s2) * 0.1):
            alphabet = "na"
            if len(sequence) % 3 == 0:
                alphabet = "codons"
        else:
            alphabet = "aa"

        return alphabet

    def __call__(self, sequence ):
        """mask a sequence."""

        sequence = re.sub("\s", "", sequence )

        a = self.getAlphabet( sequence )

        seq = list(sequence)

        if len(seq) < 5:
            # do not mask empty/short sequences
            pass
        elif a == "aa" and self.mHasPeptideMasking:
            c = 0
            for p, m in zip( sequence, self.maskSequence( sequence ) ):
                if m in "Xx":
                    if p.isupper():
                        seq[c] = "X"
                    else:
                        seq[c] = "x"
                c += 1

        elif a == "codons" and self.mHasPeptideMasking:

            peptide_sequence = Genomics.TranslateDNA2Protein( sequence )
            masked_sequence = self.maskSequence( peptide_sequence )
            
            c = 0
            for p, m in zip(peptide_sequence, masked_sequence):
                if m in "Xx":
                    if p.isupper():
                        seq[c:c+3] = [ "N" ] * 3
                    else:
                        seq[c:c+3] = [ "n" ] * 3
                c += 3

        elif a in ("na", "codons") and self.mHasNucleicAcidMasking:
            return self.maskSequence( sequence )
        else:
            raise ValueError("masking of sequence type %s not implemented." % a)

        return "".join( seq )

    def maskSequence( self, peptide_sequence ):
        """mask peptide sequence
        """
        
        Masker.__init__(self)

        outfile, filename_peptide = tempfile.mkstemp()
        os.write(outfile, ">test\n%s\n" % (peptide_sequence))
        os.close(outfile)

        infile = filename_peptide
        statement = self.mCommand % locals()

        E.debug( "statement: %s" % statement )

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise RuntimeError("Error in running %s \n%s\nTemporary directory" % (statement, err))

        os.remove( filename_peptide ) 
        
        masked_sequence = re.sub("\s", "", string.join(out.split("\n")[1:], ""))
        
        return masked_sequence

    def maskSequences( self, sequences ):
        '''mask a collection of sequences.'''

        outfile, infile = tempfile.mkstemp()

        for x,s in enumerate(sequences):
            os.write(outfile, ">%i\n%s\n" % (x,s) )
                     
        os.close(outfile)
                     
        statement = self.mCommand % locals()

        E.debug( "statement: %s" % statement )

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise RuntimeError("Error in running %s \n%s\nTemporary directory" % (statement, err))

        result = [ x.sequence for x in FastaIterator.iterate( StringIO.StringIO( out) ) ]

        os.remove( infile )
        
        return result
        

class MaskerBias (Masker): 

    mCommand = "biasdb.pl %(infile)s"
    mHasPeptideMasking = True

class MaskerSeg (Masker):

    # mCommand = "seg %(infile)s 12 2.2 2.5 -x"
    mCommand = "segmasker -in %(infile)s -window 12 -locut 2.2 -hicut 2.5 -outfmt fasta"
    mHasPeptideMasking = True

class MaskerDustMasker( Masker ):
    '''use dustmasker. masked chars are returned as 
    lower case characters.'''
    mCommand = "dustmasker -outfmt fasta -in %(infile)s"
    mHasNucleicAcidMasking = True

class MaskerRandom (Masker): 
    """randomly mask a proportion of positions in a sequence in multiple alignment."""

    def __init__(self, proportion = 10, *args, **kwargs):
        Masker.__init__(self, *args, **kwargs)
        self.mProportion = proportion / 100.0

    def __call__(self, sequence ):
        """mask a sequence."""

        sequence = re.sub("\s", "", sequence )

        a = self.getAlphabet( sequence )

        if a == "codons":
            frame = 3
        else: 
            frame = 1

        positions = [ x for x in range(0,len(sequence), frame) if sequence[x] != "-" ]
        to_mask = random.sample( positions, int(len(positions) * self.mProportion) )
        print int(len(positions) * self.mProportion)
        
        s = list(sequence)
        print to_mask
        for x in to_mask:
            for y in range(x, x+frame):
                s[x] == "x"

        return "".join( s )

if __name__ == "__main__":
    x = MaskerRandom()
    print x("AAA AAA AAA AAA --- AAA AAA AAA AAA" )
        
    x = MaskerDustMasker()
    print x.maskSequences( ("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
                            "GGGGGGGGGG", ) )
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Maq.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import copy, string, collections

class Error(Exception):
    """Base class for exceptions in this module."""
    def __str__(self):
        return str(self.message)
    def _get_message(self, message): return self._message
    def _set_message(self, message): self._message = message
    message = property(_get_message, _set_message)

class ParsingError(Error):
    """Exception raised for errors while parsing

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message, line = None):
        if line:
            self.message = message + " at line:" + line
        else:
            self.message = message

class Match:
    """a maq match.
    """
    
    def __init__(self):
        self.mName = None
        self.contig = None
        self.start = 0
        self.strand = "+"
        self.mPairInsert = 0
        self.mPairFlag = 0
        self.mQuality = 0
        self.mQualitySingle = 0
        self.mQualityAlternative = 0
        self.mNMismatches = 0
        self.mMismatchQuality = 0
        self.mNMismatch0 = 0
        self.mNMismatch1 = 0
        self.mLength = 0

    def __str__(self):
        return "\t".join( map(str, (self.mName,
             self.contig,
             self.start+1,
             self.strand,
             self.mPairInsert,
             self.mPairFlag,
             self.mQuality,
             self.mQualitySingle,
             self.mQualityAlternative,
             self.mNMismatches,
             self.mMismatchQuality,
             self.mNMismatch0,
             self.mNMismatch1,
             self.mLength ) ))

    def fromTable( self, data ):
        
        if len(data) == 14:
            (self.mName,
             self.contig,
             self.start,
             self.strand,
             self.mPairInsert,
             self.mPairFlag,
             self.mQuality,
             self.mQualitySingle,
             self.mQualityAlternative,
             self.mNMismatches,
             self.mMismatchQuality,
             self.mNMismatches0,
             self.mNMismatches1,
             self.mLength ) = data
            
        ( self.start,   
          self.mPairInsert,
          self.mPairFlag,
          self.mQuality,
          self.mQualitySingle,
          self.mQualityAlternative,
          self.mNMismatches,
          self.mMismatchQuality,
          self.mNMismatches0,
          self.mNMismatches1,
          self.mLength ) = map( int, ( self.start,   
                                       self.mPairInsert,
                                       self.mPairFlag,
                                       self.mQuality,
                                       self.mQualitySingle,
                                       self.mQualityAlternative,
                                       self.mNMismatches,
                                       self.mMismatchQuality,
                                       self.mNMismatches0,
                                       self.mNMismatches1,
                                       self.mLength ) )
        ## maq starts counting at 1
        self.start -= 1
             
def iterator( infile ):
    """iterate over the contents of a maq file.
    """
    while 1:
        line = infile.readline()
        if not line: raise StopIteration
        if line[0] == "#": continue
        if line.startswith( "read\tcontig"): continue
        match = Match()
        match.fromTable( line[:-1].split() )
        yield match



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
rnaseq_bam_vs_exons.py - compare bam file against exons
=======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Compare RNASeq reads in a BAM file against reference exons.

The script outputs counts for exon over-/underrun. 

For unspliced reads, any bases extending beyond exon boundaries are counted. 

For spliced reads, both parts of the reads are examined for their overlap.
As a consequence, counts are doubled for spliced reads.

This script is for validation purposes:

   * Exon overrun should be minimal - reads should not 
      extend beyond known exons.

   * Spliced reads should link known exons.

The script requires a list of non-overlapping exons.



Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

For read counts to be correct the NH flag to be set correctly.

Code
----

'''

import os
import sys
import re
import optparse
import collections
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam
import CGAT.GTF as GTF

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-e", "--filename-exons", dest="filename_exons", type="string",
                       help = "gtf formatted file with exons locations. [%default]" )

    parser.set_defaults(
        filename_exons = None,
        read_length = 200,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    exons = GTF.readAndIndex( GTF.iterator( IOTools.openFile( options.filename_exons ) ) )

    pysam_in = pysam.Samfile( "-", "rb" )
    
    nspliced = 0
    nspliced_ignored = 0
    nspliced_nooverlap = 0
    nspliced_halfoverlap = 0
    nspliced_bothoverlap = 0
    nspliced_overrun = [0] * 2 * (options.read_length + 10)
    nspliced_exact = 0
    nspliced_inexact = 0
    nunspliced = 0
    nunspliced_overlap = 0
    nunspliced_ignored = 0
    nunspliced_nooverlap = 0
    nunspliced_overrun = [0] * (options.read_length + 10)

    overrun_offset = options.read_length + 10

    ninput = 0
    nunmapped = 0

    c = E.Counter()

    def _splice_overrun( start, end, overlap ):
        '''return splicesite over/underrun.

        positive values: overrun
        negative values: underrun
        0: no over/underrun
        '''

        exon_start = min( [x[0] for x in overlap] )
        exon_end = max( [x[1] for x in overlap] )

        if start <= exon_start and end > exon_start:
            # overrun at start or match
            r = exon_start - start
        elif start < exon_end and end >= exon_end:
            # overrun at end or match
            r = end - exon_end
        else:
            # underrun - distance to closest exon boundary
            r = -min( start - exon_start, exon_end - end )

        return r

    for read in pysam_in:
        ninput += 1
        if read.is_unmapped: 
            nunmapped += 1
            continue
        
        # check for BAM_CREF_SKIP code in cigar string
        cigar = read.cigar
        is_spliced = 3 in [ x[0] for x in cigar ]

        contig = pysam_in.getrname( read.tid ) 
        start = read.pos
        end = read.aend
        if is_spliced:
            # count both ends
            nspliced += 1

            if len(cigar) != 3:
                nspliced_ignored += 1
                continue

            start5, end5 = start, start + cigar[0][1]
            start3, end3 = end - cigar[2][1], end
            try:
                overlap3 = list( exons.get( contig, start3, end3 ) )
                overlap5 = list( exons.get( contig, start5, end5 ) )
            except KeyError:
                overlap3 = overlap5 = []

            ovl3 = len(overlap3) 
            ovl5 = len(overlap5) 
            o3 = o5 = None
            if not ovl3 and not ovl5:
                nspliced_nooverlap += 1
            elif ovl3 and not ovl5:
                nspliced_halfoverlap += 1
                o3 = _splice_overrun( start3, end3, overlap3 )
            elif ovl5 and not ovl3:
                nspliced_halfoverlap += 1
                o5 = _splice_overrun( start5, end5, overlap5 )
            else:
                # both overlap
                nspliced_bothoverlap += 1
                o3 = _splice_overrun( start3, end3, overlap3 )
                o5 = _splice_overrun( start5, end5, overlap5 )

            if o3 != None:
                if o3 == 0: nspliced_exact += 1
                else: nspliced_inexact += 1
                nspliced_overrun[ max(0, overrun_offset + o3)] += 1
            if o5 != None:
                if o5 == 0: nspliced_exact += 1
                else: nspliced_inexact += 1
                nspliced_overrun[ max(0, overrun_offset + o5)] += 1
        else:
            nunspliced += 1
            try:
                overlap = list( exons.get( contig, start, end ) )
            except KeyError:
                overlap = []

            if len(overlap) == 0:
                nunspliced_nooverlap += 1
            elif len(overlap) >= 1:
                nunspliced_overlap += 1
                # multiple overlap - merge exons (usually: small introns)
                exon_start = min( [x[0] for x in overlap] )
                exon_end = max( [x[1] for x in overlap] )
                ostart = max( 0, exon_start - start )
                oend = max( 0, end - exon_end )
                o = min(end, exon_end) - max( start, exon_start)
                overrun = ostart + oend
                nunspliced_overrun[overrun] += 1

    # output histograms
    outfile = E.openOutputFile( "overrun" )
    outfile.write( "bases\tunspliced_overrun_counts\tspliced_overrun_counts\tspliced_underrun_counts\n" )
    _nspliced_overrun = nspliced_overrun[overrun_offset:]
    _nspliced_underrun = nspliced_overrun[:overrun_offset+1]
    _nspliced_underrun.reverse()
    for x, v in enumerate( zip( nunspliced_overrun, _nspliced_overrun, _nspliced_underrun) ):
        outfile.write( "%i\t%s\n" % (x, "\t".join(map(str,v))))
    outfile.close()

    # output summary
    # convert to counter
    c.input = ninput
    c.unmapped = nunmapped
    c.mapped = ninput - nunmapped

    c.unspliced = nunspliced
    c.unspliced_nooverlap = nunspliced_nooverlap
    c.unspliced_nooverrun = nunspliced_overrun[0]
    c.unspliced_overlap = nunspliced_overlap
    c.unspliced_overrun = sum( nunspliced_overrun[1:])

    c.spliced = nspliced
    c.spliced_nooverlap = nspliced_nooverlap
    c.spliced_halfoverlap = nspliced_halfoverlap
    c.spliced_bothoverlap = nspliced_bothoverlap
    c.spliced_exact = nspliced_exact
    c.spliced_inexact = nspliced_inexact
    c.spliced_ignored = nspliced_ignored
    c.spliced_underrun = sum( _nspliced_underrun[1:])
    c.spliced_overrun = sum( _nspliced_overrun[1:])
    
    outfile = options.stdout
    outfile.write( "category\tcounts\n" )
    for k,v in c.iteritems():
        outfile.write( "%s\t%i\n" % (k,v))
    
    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
nucmer2bed.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes as input the output from nucmer 'show-coords' and outputs
a bed file with alignment intervals.

Usage
-----

Example::

   python nucmer2bed.py --help

Type::

   python nucmer2bed.py --help

for command line help.

Documentation
-------------

Read from stdin and output to stdout. Requires that the -T option in 
show-coords was used to produce tabular output.

Code
----

'''

import os
import sys
import re
import optparse

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-t", "--type", dest="type", type="choice",
                      choices = ("bed", "bed4"), help="output bed type: bed or bed4"  )
    
    parser.set_defaults(type="bed")

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )



    # read in header lines
    inf = sys.stdin
    input_files = inf.readline()
    program = inf.readline()
    spacer = inf.readline()
    header = inf.readline()
    
    # contig is the reference tag


    # get the coordinates for the alignment with respect to
    # the reference
    for line in inf.readlines():
        data = line[:-1].split("\t")
        contig, start, end = data[-2], data[0], data[1]
        if options.type == "bed4":
            name = data[-1]
            options.stdout.write("\t".join(map(str,[contig, start, end, name])) + "\n")
        else:
            options.stdout.write("\t".join(map(str,[contig, start, end])) + "\n")

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )


#################################################################
# convert nucmer output coords file to tab delimited text file
#################################################################
import sys
import os
import re

inf = sys.stdin

input = inf.readline()
program = inf.readline()
spacer = inf.readline()

header = inf.readline()
headers = "\t".join(["s1", "e1", "s2", "e2", "len1", "len2", "%id", "len_r", "len_q", "ref_tag", "query_tag"])
sys.stdout.write("%s\n" % headers)
equals_spacer = inf.readline()
for line in inf.readlines():
    data = [x for x in line[:-1].split(" ") if x and x != "|"]
    sys.stdout.write("\t".join(map(str,data)) + "\n")



################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree_diff.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree_diff.py - compare two sets of trees
========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This scripts reads two collections of trees and checks if the
trees in the two collections are identical.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----
"""
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess

from types import *

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree_diff.py 2782 2009-09-10 11:40:29Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option("-1", "--filename-tree1", dest="filename_tree1", type="string",
                      help="filename with first tree(s)."  )
    parser.add_option("-2", "--filename-tree2", dest="filename_tree2", type="string",
                      help="filename with second tree(s)."  )
    parser.add_option("-o", "--outgroup", dest="outgroup", type="string",
                      help="reroot with outgroup before processing.")

    parser.set_defaults(
        filename_tree1 = None,
        filename_tree2 = None,
        outgroup = None
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if (len(args) == 2):
        options.filename_tree1, options.filename_tree2 = args

    if not options.filename_tree1 or not options.filename_tree2:
        raise ValueError( "please specify two trees." )
    
    ## take first trees
    nexus = TreeTools.Newick2Nexus( open(options.filename_tree1, "r") )
    trees1 = nexus.trees
    if options.loglevel >= 1:
        options.stdlog.write( "# read %i trees from %s.\n" % (len(trees1), options.filename_tree1))

    ## take first trees
    nexus = TreeTools.Newick2Nexus( open(options.filename_tree2, "r") )
    trees2 = nexus.trees
    if options.loglevel >= 1:
        options.stdlog.write( "# read %i trees from %s.\n" % (len(trees2), options.filename_tree2))
        
    ntotal, nsame, ndiff = 0,0,0

    if options.outgroup:
        for tree in trees1:
            tree.root_with_outgroup( options.outgroup )
        for tree in trees2:
            tree.root_with_outgroup( options.outgroup )
    
    for x in range(len(trees1)):
        for y in range(len(trees2)):
            if options.loglevel >= 2:
                print trees1[x]
                print trees2[y]
            if trees1[x].is_identical(trees2[y]):
                code = "="
                nsame += 1
            else:
                code = "<>"
                ndiff += 1
            options.stdout.write("%s\t%i\t%i\n" % (code, x, y ))
            ntotal += 1
            
    options.stdlog.write( "# n1=%i, n2=%i, ntotal=%i, nsame=%i, ndiff=%i\n" % (len(trees1), len(trees2), ntotal, nsame, ndiff))
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gtfs2graph.py - compute overlap graph between genes
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script compares two :term:`gtf` output files and outputs a bi-partite
graph connecting overlapping genes/exons/...

This script needs some work, only gene-gene output works.

Usage
-----

Example::

   python gtfs2graph.py --help

Type::

   python gtfs2graph.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import collections

USAGE="""python %s [OPTIONS] input1 input2


Version: $Id: gtfs2graph.py 2781 2009-09-10 11:33:14Z andreas $
""" % sys.argv[0]


import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import bx.intervals.io
import bx.intervals.intersection
import numpy

class Counter:

    mPercentFormat = "%5.2f"

    def __init__(self, outfile):
        self.mOutfile = outfile
        
    def write( self, *args):
        self.mOutfile.write( "\t".join( args ) + "\n" )

    def getHeader( self ):
        h = []
        for a in ("genes", "exons", "bases" ):
            for b in ("total", "ovl", "unique"):
                for c in ("1","2"):
                    h.append( "n" + a + "_" + b + c )
        for a in ("genes", "exons", "bases" ):
            for b in ("ovl", "unique"):
                for c in ("1","2"):
                    h.append( "p" + a + "_" + b + c )

        return "\t".join(h)

    @E.cachedmethod
    def buildIndex( self, filename ):
        """read and index."""

        idx = {}
        infile = open(filename, "r")
        for e in GTF.readFromFile(infile):
            if e.contig not in idx: idx[e.contig] = bx.intervals.intersection.Intersecter()
            idx[e.contig].add_interval( bx.intervals.Interval(e.start,e.end,value=e) )
        infile.close()
        return idx

    def _count( self, filename, idx ):

        overlapping_genes = set()
        genes = set()
        # iterate over exons
        infile = open( filename, "r" )
        it = GTF.iterator( infile )

        nexons, nexons_overlapping = 0, 0
        nbases, nbases_overlapping = 0, 0
        for this in it:
            nexons += 1
            nbases += this.end - this.start 
            genes.add( this.gene_id )
            
            try:
                intervals = idx[this.contig].find( this.start, this.end )
            except KeyError:
                continue


            if len(intervals) == 0: 
                continue

            overlapping_genes.add( this.gene_id )
            nexons_overlapping += 1
            start, end = this.start, this.end 
            counts = numpy.zeros( end - start, numpy.int )
            for other in intervals:
                for x in range( max(start, other.start ) - start, min(end, other.end ) - start ): 
                    counts[x] += 1
            nbases_overlapping += sum( [1 for x in counts if x > 0 ] )

        infile.close()

        return len(genes), len( overlapping_genes), nexons, nexons_overlapping, nbases, nbases_overlapping

    def count( self, filename1, filename2 ):
        """count overlap between two gtf files."""

        E.info( "counting started for %s versus %s" % (filename1, filename2))

        idx2 = self.buildIndex( filename2 )

        (self.mGenes1, self.mGenesOverlapping1,
         self.mExons1, self.mExonsOverlapping1,
         self.mBases1, self.mBasesOverlapping1 ) = \
         self._count( filename1, idx2 )

        self.mGenesUnique1 = self.mGenes1 - self.mGenesOverlapping1
        self.mExonsUnique1 = self.mExons1 - self.mExonsOverlapping1
        self.mBasesUnique1 = self.mBases1 - self.mBasesOverlapping1

        idx1 = self.buildIndex( filename1 )

        (self.mGenes2, self.mGenesOverlapping2,
         self.mExons2, self.mExonsOverlapping2,
         self.mBases2, self.mBasesOverlapping2 ) = \
         self._count( filename2, idx1 ) 

        self.mGenesUnique2 = self.mGenes2 - self.mGenesOverlapping2
        self.mExonsUnique2 = self.mExons2 - self.mExonsOverlapping2
        self.mBasesUnique2 = self.mBases2 - self.mBasesOverlapping2

    def __str__(self):

        return "\t".join( map( str, (
                        self.mGenes1, self.mGenes2,
                        self.mGenesOverlapping1, self.mGenesOverlapping2,
                        self.mGenesUnique1, self.mGenesUnique2,
                        self.mExons1, self.mExons2,
                        self.mExonsOverlapping1, self.mExonsOverlapping2,
                        self.mExonsUnique1, self.mExonsUnique2,
                        self.mBases1, self.mBases2,
                        self.mBasesOverlapping1, self.mBasesOverlapping2,
                        self.mBasesUnique1, self.mBasesUnique2 ) ) ) + "\t" +\
              "\t".join( map( lambda x: IOTools.prettyPercent( *x), (
                    (self.mGenesOverlapping1, self.mGenes1),
                    (self.mGenesOverlapping2, self.mGenes2),
                    (self.mGenesUnique1, self.mGenes1), 
                    (self.mGenesUnique2, self.mGenes2),
                    (self.mExonsOverlapping1, self.mExons1), 
                    (self.mExonsOverlapping2, self.mExons2),
                    (self.mExonsUnique1, self.mExons1), 
                    (self.mExonsUnique2, self.mExons2),
                    (self.mBasesOverlapping1, self.mBases1), 
                    (self.mBasesOverlapping2, self.mBases2),
                    (self.mBasesUnique1, self.mBases1), 
                    (self.mBasesUnique2, self.mBases2)  ) ) )

class CounterGenes(Counter):
    """output only genes."""
    
    mSeparator = ";"

    def __init__(self, *args, **kwargs ):
        Counter.__init__(self, *args, **kwargs)

    def getHeader( self ):
        h = [ "genes", "gene2" ]
        return "\t".join(h)

    def _run( self, filename, idx ):

        # iterate over exons
        infile = open( filename, "r" )
        it = GTF.iterator( infile )

        keys = set()

        for this in it:
            
            try:
                intervals = idx[this.contig].find( this.start, this.end )
            except KeyError:
                continue

            if len(intervals) == 0: 
                continue

            for i in intervals:
                key = "%s-%s" % (this.gene_id, i.value.gene_id )
                if key not in keys:
                    self.write( this.gene_id, i.value.gene_id )
                    keys.add( key )

        infile.close()


    def run( self, filename1, filename2 ):
        """count overlap between two gtf files."""

        E.info( "counting started for %s versus %s" % (filename1, filename2))

        idx2 = self.buildIndex( filename2 )
        self._run( filename1, idx2 )

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gtfs2graph.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-s", "--ignore-strand", dest="ignore_strand", action="store_true",
                      help="ignore strand information [default=%default]." )

    parser.add_option("-u", "--update", dest="filename_update", type="string",
                      help="if filename is given, previous results will be read from there and only changed sets will be computed [default=%default]." )

    parser.add_option("-p", "--pattern-id", dest="pattern_id", type="string",
                      help="pattern to convert a filename to an id [default=%default]." )

    parser.add_option("-g", "--genes", dest="genes", action="store_true",
                      help="only output gene stats (includes gene lists) [default=%default]." )

    parser.set_defaults(
        ignore_strand = False,
        filename_update = None,
        pattern_id = "(.*).gtf",
        genes = False,
        )

    (options, args) = E.Start( parser )

    if len(args) != 2:
        print USAGE
        raise ValueError( "two arguments are required" )

    if options.genes:
        counter = CounterGenes( options.stdout )
    else:
        counter = Counter( options.stdout )

    options.stdout.write( counter.getHeader() + "\n" )

    counter.run( args[0], args[1] )
    
    E.Stop()    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperSlr.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse, shutil

from types import *

"""Wrapper for CodeSlr
"""

import Experiment
import Mali
import TreeTools

class Error(Exception):
    """Base class for exceptions in this module."""
    pass

    def __str__(self):
        return str(self.message)

class ParsingError(Error):
    """Exception raised for errors while parsing

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message, line):
        self.message = message + " at line " + line

class UsageError(Error):
    """Exception raised for errors while starting

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class Result:
    
    mMaxLineLength = 100
    
    def __init__(self):
        self.mWarnings = []

    def truncateLine( self, line ):
        if len(line) > self.mMaxLineLength:
            return "%s..." %  line[:self.mMaxLineLength]
        else:
            return line
        
    def __str__(self):

        s = []
        members = self.__dict__
        keys = list(members.keys())
        keys.sort()
        
        for key in keys:
            
            if key[0] == 'm':

                m = members[key]
                if type(m) in (ListType, TupleType):
                    for x in range(len(m)):
                        s.append( ("%-40s: %s" % (key, self.truncateLine(str(m[x]))) ))
                elif type(m) in (DictType,):
                    for x, y in m.items():
                        s.append( ("%-40s: %s: %s" % (key, str(x), self.truncateLine(str(y)))))
                else:
                    s.append( ("%-40s: %s" % (key, self.truncateLine(str(m)))))
                    
        return "\n".join(s)

class SlrResult(Result):
    """result of an SLR run."""
    def __init__(self):
        Result.__init__(self)
        self.mSites = []
        self.mNSitesSynonymous = 0
        self.mNSitesGaps = 0
        self.mNSitesSingleChar = 0                
        self.mLog = ""
        self.mResult = ""
    
class SlrResultSite(Result):
    """result with site specific information.

    mResult can be either
    -,--        for negatively selected sites
    0           for neutral sites
    +,++        for positively selected sites
    None        for sites, which can't be estimated: gapped sites/synonymous sites/etc.
    """
    def __init__(self,
                 residue, lneutral, loptimal, w,
                 statistic, pvalue, adjusted_pvalue,
                 result, warning, note, is_synonymous):
        
        Result.__init__(self)        
        self.mResidue = residue
        self.mLikelihoodNeutral = lneutral
        self.mLikelihoodOptimal = loptimal
        self.mOmega = w
        self.mStatistic = statistic
        self.mPValue = pvalue
        self.mAdjustedPValue = adjusted_pvalue

        self.mResult = result
        self.mIsSynonymous = is_synonymous
        self.mHasWarning = warning 
        self.mNote = note

        self.mIsSynonymous = note == "Synonymous"
        
        if self.mResult:
            self.mIsPositive = self.mResult[0] == "+"
            self.mIsNegative = self.mResult[0] == "-"
        else:
            self.mIsPositive = False
            self.mIsNegative = False            
        
    def __str__(self):
        return "\t".join( ( "%i" % self.mResidue,
                            "%s" % self.mResult,
                            "%s" % str(self.mHasWarning),
                            "%5.2f" % self.mOmega,
                            "%5.2f" % self.mPValue,
                            "%5.2f" % self.mAdjustedPValue,                            
                            "%5.2f" % self.mLikelihoodNeutral,
                            "%5.2f" % self.mLikelihoodOptimal,
                            "%5.2f" % self.mStatistic,
                            "%s" % self.mNote) )

    def isSynonymous(self ):
        return self.mIsSynonymous
    
    def isNegative(self, threshold = 0.05, use_adjusted = False):
        """returns true, if site is under negative selection."""
        if not self.mIsNegative:
            return False
            
        if use_adjusted:
            p = self.mAdjustedPValue
        else:
            p = self.mPValue

        return p < threshold 

    def isPositive(self, threshold = 0.05, use_adjusted = False):
        """returns true, if site is under positive selection."""
        
        if not self.mIsPositive:
            return False
            
        if use_adjusted:
            p = self.mAdjustedPValue
        else:
            p = self.mPValue

        return p < threshold 
            
class Slr:
    """
    """
    mExecutable = "Slr"

    def __init__( self ):

        self.mFilenameTree = None
        self.mFilenameSequences = None

        self.mDefaults = { "positive_only": "0",
                           "initial_kappa": "2.0",
                           "initial_omega": "0.1" }

        self.mOptions = {
            "positive_only" : {  "0": "output positve and neutral sites",
                                 "1": "only output positive sites"
                                 },
            "initial_kappa"   : { "#" : "initial value for kappa",
                                  },
            "initial_omega"  : { "#" : "initial value for omega",
                                 },
            }

    def GetOptions( self ):
        """return options in pretty format"""
        result = ["# Options for Slr" ]
        for var in self.mOptions.keys():
            result.append("# %-40s: %s" % (var, self.mOptions[var]))
        return string.join(result, "\n")

    ##------------------------------------------------------------------------
    def WriteAlignment( self, mali ):
        """write alignment in Phylip format."""

        mali.mapIdentifiers( self.mMapOld2New )
        outfile = open( self.mTempdir + "/" + self.mFilenameSequences, "w" )
        mali.writeToFile( outfile, format="phylip" )
        outfile.close()

    ##------------------------------------------------------------------------
    def WriteTree( self, tree ):
        """write tree to file.
        """

        nexus = TreeTools.Newick2Nexus( tree )
        t = nexus.trees[0]
        TreeTools.MapTaxa( t, self.mMapOld2New )
        
        outfile = open( self.mTempdir + "/" + self.mFilenameTree, "w" )
        outfile.write("%i 1\n" % self.mNumSequences )
        outfile.write("%s\n" % TreeTools.Tree2Newick(t))
        outfile.close()

    ##------------------------------------------------------------------------
    def BuildMap( self, mali ):
        """build map of identifiers.
        SLR requires the taxa in the tree to be identical to the
        position (not the name) in the multiple alignment file (and
        not the ids). Thus choose sequential numbering for ids.
        """
        
        self.mMapOld2New = {}
        self.mMapNew2Old = {}
        x = 0
        for id in mali.getIdentifiers():
            x += 1            
            new_id = "%i" % x
            self.mMapOld2New[id] = new_id
            self.mMapNew2Old[new_id] = id

    ##------------------------------------------------------------------------        
    def Run( self, alignment, tree = None,
             dump = 0,
             test = False,
             options = {} ):

        self.mTempdir = tempfile.mkdtemp()
        self.mFilenameSequences = "input"

        self.mFilenameOutput = "output"        
        self.mNumSequences = len(alignment)

        self.BuildMap( alignment )
        
        self.WriteAlignment( alignment )

        self.mNumSequenes = alignment.getLength()
        
        if test:
            print "# temporary directory is %s" % self.mTempdir
            
        if tree:
            self.mFilenameTree = "tree"
            
            ## check what kind of tree is given.
            if type(tree) == StringType:
                t = tree.strip()
                if not (t[0] == "(" and t[-1] in ");"):
                    tree = "".join(open( tree, "r").readlines())
                self.WriteTree( tree )                    
            
        self.mFilenameControl = "%s.ctl" % self.mExecutable

        ## build options
        ## write set options
        set_options = {}
        for key, vv in options.items():

            if key not in self.mOptions:
                raise IndexError, "illegal option %s" % key

            values = vv.strip().split(" ")
            
            descriptions = []
            for value in values:
                if value in self.mOptions[key]:
                    descriptions.append( self.mOptions[key][value] )
                elif "#" in self.mOptions[key]:
                    try:
                        v = float(value)
                    except ValueError:
                        raise ValueError, "not a number: %s for option %s" % (value, key)
                    descriptions.append( self.mOptions[key]["#"] )
                else:
                    raise ValueError, "illegal value %s for option %s: possible values are: %s" % (value, key,
                                                                                                   " ".join(self.mOptions[key].keys()) )

            set_options[key] = vv


        ## write default options
        for key, vv in self.mDefaults.items():
            
            if key in set_options: continue

            if key not in self.mOptions:
                raise IndexError, "illegal option %s" % key

            values = vv.strip().split(" ")
            
            descriptions = []
            for value in values:
                if value in self.mOptions[key]:
                    descriptions.append( self.mOptions[key][value] )
                elif "#" in self.mOptions[key]:
                    try:
                        v = float(value)
                    except ValueError:
                        raise ValueError, "not a number: %s for option %s" % (value, key)
                    descriptions.append( self.mOptions[key]["#"] )
                else:
                    raise ValueError, "illegal value %s for option %s: possible values are: %s" % (value, key,
                                                                                                   " ".join(self.mOptions[key].keys()) )

            set_options[key] = vv

        # write a dummy control file
        outfile = open( self.mTempdir + "/slr.ctl", "w" )
        outfile.write("\n")
        outfile.close()
            
        statement = " ".join( ( self.mExecutable,
                                "-seqfile %s" % self.mFilenameSequences,
                                "-treefile %s" % self.mFilenameTree,
                                "-outfile %s" % self.mFilenameOutput,
                                "-reoptimize 1", 
                                "-positive_only %s" % set_options["positive_only"],
                                "-kappa %s" % set_options["initial_kappa"],
                                "-omega %s" % set_options["initial_omega"] ) )        

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempdir,
                              close_fds = True)                              

        (out, err) = s.communicate()
        
        if s.returncode != 0:
            raise UsageError, "Error in running %s \n%s\n%s\nTemporary directory in %s" % (statement, err, out, self.mTempdir)

        if dump:
            print "# stdout output of %s:\n%s\n######################################" % (self.mExecutable, out)

        lines = open( "%s/%s" % (self.mTempdir, self.mFilenameOutput), "r").readlines()

        if len(lines) == 0:
            raise UsageError, "Empty result from %s \n%s\n%s\nTemporary directory in %s" % (statement, err, out, self.mTempdir)            

        if dump: 
            print "# result output of %s:\n%s\n######################################" % (self.mExecutable, "".join(lines))        

        if not test:
            shutil.rmtree( self.mTempdir )

        return self.parseOutput( lines, out.split("\n") )

    def nextSection( self, lines):
        if lines:
            while not lines[0]: del lines[0]
        
    def skipSection( self, lines ):

        while lines[0]: del lines[0]
        while not lines[0]: del lines[0]

    def parseOutput( self, lines, lines_log = None ):
        """parse Slr output.

        if lines_log is given, the output from the logfile is parsed
        as well.
        """
        
        result = SlrResult()
        
        # chop
        lines = map(lambda x: x[:-1], lines )
        result.mResult = "\n".join(lines)

        self.parseSites( lines, result )
        
        if lines_log:
            if lines_log[0][-1] == "\n":
                lines_log = map(lambda x: x[:-1].strip(), lines_log )
            result.mLog = "\n".join(lines_log)
            self.parseLog( lines_log, result )

        return result        

    def parseLog( self, lines, result ):
        """parse log file.
        """
        ## ignore everything before the results section
        ## (that includes the supplied parameters)

        while not re.match("\*\*\*", lines[0]):
            del lines[0]
        del lines[0]

        result.mLogLikelihood = float(re.match("# lnL = (\S+)", lines[0]).groups()[0])
        del lines[0]            

        result.mTree = lines[0]
        del lines[0]

        del lines[0]

        if re.search("nan", lines[0]):
            result.mKappa, result.mOmega = 0, 0
        else:
            try:
                result.mKappa, result.mOmega = map(float, re.match("# Kappa\s+=\s+(\S+)\s*Omega\s+=\s+(\S+)", lines[0]).groups())
            except AttributeError:
                raise ParsingError( "parsing error for kappa and omega", lines[0] )
                
        del lines[0]

        if re.search("nan", lines[0]):
            result.mTreeLength, result.mAverageBranchLength, \
            result.mMinBranchLength, result.mMaxBranchLength = 0, 0, 0, 0
        else:
            try:
                result.mTreeLength, result.mAverageBranchLength, \
                                    result.mMinBranchLength, result.mMaxBranchLength = map(float, \
                re.match("# Tree length =\s*(\S+),\s*average branch length\s*=\s*(\S+)\s*\(min=(\S+),\s*max=(\S+)\)", lines[0]).groups())
            except AttributeError:
                raise ParsingError( "parsing error for tree", lines[0] )
        
        del lines[0]

        result.mScaleFactor = float(re.match("# Scaling tree to neutral evolution. Factor =\s*(\S+)", lines[0]).groups()[0])

        while not re.match("# Significance", lines[0]) : del lines[0]
        del lines[0]

        result.mNPositiveSites = {}

        for x in range(4):
            try:
                cat, n1, n2 = re.match("# (.+)\s+(\d+)\s+(\d+)", lines[0]).groups()
            except AttributeError:
                raise ParsingError( "parsing for categories", lines[0] )                
            del lines[0]
            result.mNPositiveSites[cat.strip()] = (int(n1), int(n2))

        result.mNNegativeSites = {}
        
        while not re.match("# Significance", lines[0]) : del lines[0]
        del lines[0]

        for x in range(4):
            cat, n1, = re.match("# (.+)\s+(\d+)", lines[0]).groups()
            del lines[0]
            result.mNNegativeSites[cat.strip()] = int(n1)
        
    def parseSites( self, lines, result ):
        """parse sites file.

        seems to be fixed with format. Note, there seems
        to be a warning field between Result and Note without
        a heading.
        """

        fields = (0, 5, 14, 23, 32, 41, 52, 63, 69, 70)
        
        for line in lines[1:]:
            data = []

            for x in range( len(fields) - 1):
                data.append( line[fields[x]:fields[x+1]] )
            if len(line) >= fields[-1]:
                data.append( line[fields[-1]:] )
            else:
                data.append( "")

            data = map( lambda x: x.strip(), data)
            ## blank out synonymous sites and those with all gaps
            is_synonymous = False
            if data[9] in ("Synonymous", "All gaps", "Single char"):
                if data[9] == "Synonymous":
                    result.mNSitesSynonymous += 1
                    is_synonymous = True
                elif data[9] == "All gaps":
                    result.mNSitesGaps += 1
                    data[7] = None                    
                elif data[9] == "Single char":
                    result.mNSitesSingleChar += 1
                    data[7] = None
            elif data[7] == "":
                data[7] = "0"
                
            site = SlrResultSite(
                residue = int(data[0]),
                lneutral = float(data[1]),
                loptimal = float(data[2]),
                w = float(data[3]),
                statistic = float(data[4]),
                pvalue = float(data[5]),
                adjusted_pvalue = float(data[6]),
                result = data[7],
                warning = data[8] == "!",
                note = data[9],
                is_synonymous = is_synonymous)
            
            result.mSites.append( site)

def getOptions( options ):
    """translate command line options to PAML options."""

    slr_options = {}
    
    if options.omega != None:
        slr_options[ "initial_omega" ] = str(options.omega)
    if options.kappa != None:
        slr_options[ "initial_kappa" ] = str(options.kappa)
    if options.positive_only != None:
        slr_options[ "positive_only" ] = str(options.positive_only)

    return slr_options

if __name__ == "__main__":
    
    parser = E.OptionParser( version = "%prog version: $Id: WrapperSlr.py 2781 2009-09-10 11:33:14Z andreas $" )

    parser.add_option("-t", "--filename-tree", dest="filename_tree", type="string",
                      help="filename with tree information."  )
    parser.add_option("-i", "--filename-sequences", dest="filename_sequences", type="string",
                      help="filename with sequences."  )
    parser.add_option("-p", "--parse-output", dest="parse_output", action="store_true",
                      help="parse output")
    parser.add_option("-l", "--filename-log", dest="filename_log", type="string",
                      help="filename for logging information."  )
    parser.add_option( "--set-omega", dest="omega", type="float",
                      help="initial omega value.")
    parser.add_option( "--set-kappa", dest="kappa", type="float",
                      help="initial kappa value.")
    parser.add_option( "--set-positive_only", dest="positive_only", type="int",
                      help="only output positive sites.")
    parser.add_option( "--dump", dest="dump", action="store_true",
                      help="dump output."  )

    parser.set_defaults(
        analysis = None,
        filename_tree = None,
        filename_sequences = "input",
        multiple_genes = False,
        parse_output = False,
        filename_log = None,
        omega = None,
        kappa = None,
        positive_only = None,
        dump = False,
        )

    (options, args) = Experiment.Start( parser )

    slr = Slr()

    slr_options = getOptions(options)

    mali = Mali.Mali()
    mali.readFromFile( open(options.filename_sequences, "r"),
                       format = "fasta")

    if not options.filename_tree:
        raise "please supply a tree."
    
    result = slr.Run( mali, options.filename_tree,
                      dump = options.dump )

    options.stdout.write( result.mResult + "\n" )

    if options.filename_log:
        outfile = open(options.filename_log, "w" )
    else:
        outfile = options.stdlog
        
    outfile.write( result.mLog + "\n" )

    if outfile != options.stdlog:
        outfile.close()
    
    Experiment.Stop()
